# -*- coding: utf-8 -*-
"""
Created on Fri Oct 30 09:43:42 2020

@author: timhe
"""

#%%
import re
import os
from osgeo import osr
import glob
from osgeo import gdal
import datetime 
import numpy as np
import pandas as pd
from pyproj import Proj, transform
import time
import shutil
import SEBAL.pySEBAL.pySEBAL_code as sebal
import watertools
import watertools.General.data_conversions as DC
import watertools.General.raster_conversions as RC
import warnings

def Find_nearest_time(Dates, Date):
    return min(Dates, key=lambda x: abs(x - Date))               
            
def Process_LS_Image(LS_file, FOLDERS, filename_DEM, UTM_Zone, Startdate_IrriEngine, GMT_offset):
    
    print("Process: %s" %LS_file) 
    
    # Define Name of Landsat Image without band number
    Name_Landsat_Image = os.path.splitext(LS_file)[0][:-4]
    Path_row = os.path.splitext(LS_file)[0].split("_")[2]
  
    # the path to the MTL file of landsat
    Landsat_meta_fileName = os.path.join(FOLDERS[0], '%s_MTL.txt' % Name_Landsat_Image)

    # read out the general info out of the MTL file
    print("collect time information from the MTL file: %s" %Landsat_meta_fileName) 
    year, DOY, hour_GTM, minutes_GTM, UTM_Zone_LS, Sun_elevation = info_general_metadata(Landsat_meta_fileName)    

    # Get startdate IrriEngine
    Date_Start = datetime.datetime.strptime(Startdate_IrriEngine, "%Y-%m-%d") + datetime.timedelta(days=1)
    Date_End = datetime.datetime.strptime(Startdate_IrriEngine, "%Y-%m-%d") - pd.DateOffset(months=3)
    Date_Start_or = Date_Start.toordinal()
    Date_End_or = Date_End.toordinal()
    
    # Get date LS in ordinal
    Date_LS = datetime.datetime.strptime("%d%03d" %(year, DOY), "%Y%j")
    Date_LS_or = Date_LS.toordinal()    
    
    if (Date_LS_or < Date_Start_or and Date_LS_or > Date_End_or):

        # Define Time parameter using year and DOY
        TIME = datetime.datetime.strptime("%d%d_%02d%02d"%(year, DOY, hour_GTM, minutes_GTM), "%Y%j_%H%M")    
        
        # Define output cloud
        filename_Cloud = os.path.join(FOLDERS[7], "LS_CLOUD_%s_%d%02d%02d.tif" %(Path_row, TIME.year, TIME.month, TIME.day))  
     
        # Check if cloud mask already exists
        if not os.path.exists(filename_Cloud):
                   
            # Get landsat number
            Landsat_nr = int(LS_file.split("_")[0][2:4])
            
            if 'shape_dem' not in locals():
                            
                print("open dem file: %s" %filename_DEM) 
                dest_dem, DEM_resh, geo, proj, ncol, nrow, shape_dem = Create_Example_UTM_File(filename_DEM, 30, UTM_Zone)
                UTM_Zone = RC.Get_epsg(dest_dem)            
                
                # Create Latitude and Longitude files in UTM
                lat_proy_UTM, lon_proy_UTM  = DEM_lat_lon(dest_dem)      
            
                # Calculate Latitude and Longitude files in WGS84
                input_projection = Proj(init="epsg:%s" %UTM_Zone)
                output_projection = Proj(init="epsg:4326")
                lon_proy, lat_proy = transform(input_projection, output_projection, lon_proy_UTM.flatten(), lat_proy_UTM.flatten())
                lon_proy.resize((nrow, ncol))
                lat_proy.resize((nrow, ncol))
     
                # Calculate aspect and slope
                slope, aspect = Calc_Gradient(DEM_resh, 30)[2:]
    
            # calculate GTM hour
            hour_loc = hour_GTM + GMT_offset
            minutes_loc = minutes_GTM
    
            # Calculate Cosinus zenith angle
            print("calculate the cosinus zenith angle") 
            cos_zn, dr = Calc_Coz_zn_dr(GMT_offset, DOY, hour_loc, minutes_loc, lon_proy, lat_proy, slope, aspect)
    
            # Define bands used for each Landsat number
            if Landsat_nr == 5 or Landsat_nr == 7:
                Bands = np.array([1, 2, 3, 4, 5, 7, 6])
            elif Landsat_nr == 8:
               Bands = np.array([2, 3, 4, 5, 6, 7, 10, 11])
            else:
                raise RuntimeError('Landsat image not supported, use Landsat 7 or 8')           
    
            # Get correction parameters LS
            
            # Open MTL landsat and get the correction parameters
            print("collect band specific information") 
            Lmin, Lmax, k1_c, k2_c = info_band_metadata(Landsat_meta_fileName, Bands)
    
            # Mean solar exo-atmospheric irradiance for each band (W/m2/microm)
            # for the different Landsat images (L5, L7, or L8)
            ESUN_L5 = np.array([1983, 1796, 1536, 1031, 220, 83.44])
            ESUN_L7 = np.array([1997, 1812, 1533, 1039, 230.8, 84.9])
            ESUN_L8 = np.array([1973.28, 1842.68, 1565.17, 963.69, 245, 82.106])   
            
            # Create MASK for no data values LS
            filename_BQA = os.path.join(FOLDERS[0], '%s_BQA.TIF' % (Name_Landsat_Image))
            LS_BQA = Open_landsat(filename_BQA, dest_dem)
            LS_BQA = np.where(LS_BQA==0, 1, LS_BQA)
            
            # Cloud Thresholds LS
            if Landsat_nr == 8:
                Cloud_Treshold = np.array([1,2,2722,2720,2724,2728,2732,3744,3748,3752,3756])
                Real_Clouds = np.array([2800,2804,2808,2812,6816,6820,6824,6828,6848,6852,6856,6860,6896,6900,6904,6908,7072,7076,7080,7084,7104,7108,7112,7116,7840,7844,7848,7852,7872,7872,7876,7880,7884]) 
            if Landsat_nr == 5 or Landsat_nr == 7:
                Cloud_Treshold = np.array([1,672,676,680,684,1696,1700,1704,1708])
                Real_Clouds = np.array([752,756,760,764]) 
               
            if Landsat_nr == 8:                
                Cloud_Buffer_Area = np.where(np.isin(LS_BQA, Real_Clouds), 0, 1)
            if Landsat_nr == 5 or Landsat_nr == 7:
                Cloud_Buffer_Area = np.where(np.isin(LS_BQA, Real_Clouds), 0, 1)
            Cloud_Buffer_Area = RC.Create_Buffer(Cloud_Buffer_Area, 3)
            Cloud_Buffer_Area = np.where(Cloud_Buffer_Area==0, 1, 0)
                
            print("create cloud array")     
            QC_mask_Cloud_first = np.where(np.isin(LS_BQA, Cloud_Treshold), 1, np.nan)
            
            # Apply buffer around pixels
            print("apply buffer of 15 pixels of 30m around clouds")                 
            QC_mask_Cloud_second = RC.Create_Buffer(Cloud_Buffer_Area, 15)
            QC_mask_Cloud_second = np.where(QC_mask_Cloud_second == 1, np.nan, 1)

            QC_mask_Cloud = (QC_mask_Cloud_first + QC_mask_Cloud_second)/2
            QC_mask_Cloud[LS_BQA==1] = np.nan
                       
            # Create 3D array to store Spectral radiance and Reflectivity for each band
            print("get reflectance and spectral radiance of the bands")              
            Reflect, Spec_Rad = Landsat_Reflect(Bands, FOLDERS[0], Name_Landsat_Image, shape_dem, QC_mask_Cloud, Lmax, Lmin, ESUN_L5, ESUN_L7, ESUN_L8, cos_zn, dr, Landsat_nr, dest_dem)
            
            # Calculate NDVI
            print("calculate spectral indices")                  
            NDVI = Calc_NDVI(Reflect)
            NDII = (Reflect[:, :, 3] - Reflect[:, :, 4])/(Reflect[:, :, 3] + Reflect[:, :, 4])
            VSDI = 1-((Reflect[:, :, 4] - Reflect[:, :, 0]) + (Reflect[:, :, 2] - Reflect[:, :, 0]))
        
            # Surface albedo:
            Surf_albedo = (0.3 * Reflect[:, :, 0] + 0.277 * Reflect[:, :, 1] +
                           0.233 * Reflect[:, :, 2] + 0.143 * Reflect[:, :, 3] +
                           0.036 * Reflect[:, :, 4] + 0.012 * Reflect[:, :, 5] -
                           0.03 ) / np.power(0.89 , 2)            
            
            # B12 of S2
            B12 = Reflect[:, :, 5]
            
            # Remove bad pixels
            NDVI[np.isnan(QC_mask_Cloud)] = np.nan
            NDII[np.isnan(QC_mask_Cloud)] = np.nan
            VSDI[np.isnan(QC_mask_Cloud)] = np.nan
            Surf_albedo[np.isnan(QC_mask_Cloud)] = np.nan
            B12[np.isnan(QC_mask_Cloud)] = np.nan    
            
            # Guess the first watermask
            water_mask_temp = np.where(NDVI < 0.0, 1.0, 0)
            
            # Calculate LAI
            FPAR, tir_emis, Nitrogen, vegt_cover, LAI, b10_emissivity = Calc_vegt_para(NDVI, water_mask_temp)
    
            # Get meteo data
            print("get meteo data from excel file")        

            RH_format_inst = os.path.join(FOLDERS[1], "Weather_Data", "Model", "GLDAS", "three_hourly", "rh_f_inst", "Hum_GLDAS-NOAH_percentage_3hour_{yyyy}.{mm:02d}.{dd:02d}_{HH:02d}00.tif")
            temp_format_inst = os.path.join(FOLDERS[1], "Weather_Data", "Model", "GLDAS", "three_hourly", "tair_f_inst", "Tair_GLDAS-NOAH_C_3hour_{yyyy}.{mm:02d}.{dd:02d}_{HH:02d}00.tif")
             
            dest_rh_inst = RC.reproject_dataset_example(RH_format_inst.format(yyyy=TIME.year, mm=TIME.month, dd=TIME.day, HH=hour_take), dest_dem, 2)
            Humidity_inst = dest_rh_inst.GetRasterBand(1).ReadAsArray()
            Humidity_inst[Humidity_inst==0] = np.nan
            Humidity_inst[np.isnan(Humidity_inst)] = np.nanmean(Humidity_inst)
            
            dest_temp_inst = RC.reproject_dataset_example(temp_format_inst.format(yyyy=TIME.year, mm=TIME.month, dd=TIME.day, HH=hour_take), dest_dem, 2)
            Temperature_inst = dest_temp_inst.GetRasterBand(1).ReadAsArray()
            Temperature_inst[Temperature_inst==0] = np.nan
            Temperature_inst[np.isnan(Temperature_inst)] = np.nanmean(Temperature_inst)
           
                      
            try: 
                
                # Calculate vapour pressure
                esat_inst = 0.6108 * np.exp(17.27 * Temperature_inst / (Temperature_inst + 237.3)) 
                eact_inst = Humidity_inst * esat_inst / 100
                
                # Define amount of bands
                if Landsat_nr == 8:
                    Bands_thermal = 2
                else:
                    Bands_thermal = 1
                
                # Calculate Surface temperature
                print("calculate lst of landsat")               
                therm_data = Landsat_therm_data(Bands, FOLDERS[0], Name_Landsat_Image, shape_dem, QC_mask_Cloud, dest_dem)      
                therm_data[therm_data==0] = np.nan
                Surface_temp, cloud_mask_temp = Calc_surface_water_temp(Temperature_inst, Landsat_nr, Lmax, Lmin, therm_data, b10_emissivity, k1_c, k2_c, eact_inst, shape_dem, water_mask_temp, Bands_thermal, 0.91 , 0.866 , 30, 1)
          
                # Define output names
                LS_fileT = os.path.join(FOLDERS[8], "LS_LST_%s_%d%02d%02d_%02d%02d.tif" %(Path_row,TIME.year, TIME.month, TIME.day, TIME.hour, TIME.minute))
                DC.Save_as_tiff(LS_fileT, Surface_temp, geo, proj)   
                
            except:
                print("Temperature is negative, so LS %s is not processed" %Name_Landsat_Image)
                
               
            Surf_albedo[Surf_albedo>1] = np.nan
            NDVI[NDVI>1] = np.nan
            VSDI[VSDI==0.] = np.nan
            VSDI[VSDI==1] = np.nan
            NDII[NDII==0.] = np.nan 

            filename_NDVI = os.path.join(FOLDERS[2], "LS_NDVI_%s_%d%02d%02d.tif" %(Path_row,TIME.year, TIME.month, TIME.day))
            filename_VSDI = os.path.join(FOLDERS[4], "LS_VSDI_%s_%d%02d%02d.tif" %(Path_row,TIME.year, TIME.month, TIME.day))
            filename_NDII = os.path.join(FOLDERS[5], "LS_NDII_%s_%d%02d%02d.tif" %(Path_row,TIME.year, TIME.month, TIME.day))
            filename_Albedo = os.path.join(FOLDERS[3], "LS_Albedo_%s_%d%02d%02d.tif" %(Path_row,TIME.year, TIME.month, TIME.day))
            filename_B12 = os.path.join(FOLDERS[6], "LS_B12_%s_%d%02d%02d.tif" %(Path_row,TIME.year, TIME.month, TIME.day))
            
            # Save NDVI, Albedo, VSDI, NDII, B12, Cloud
            print("save bands of landsat")                
            DC.Save_as_tiff(filename_Cloud, QC_mask_Cloud, geo, proj)   
            DC.Save_as_tiff(filename_NDVI, NDVI, geo, proj)      
            DC.Save_as_tiff(filename_VSDI, VSDI, geo, proj)      
            DC.Save_as_tiff(filename_NDII, NDII, geo, proj)      
            DC.Save_as_tiff(filename_Albedo, Surf_albedo, geo, proj)      
            DC.Save_as_tiff(filename_B12, B12, geo, proj)   
              
    return()  

#------------------------------------------------------------------------------
def info_general_metadata(filename):
    """
    This function retrieves general information of the Landsat image
    (date and time aquired, UTM zone, sun elevation) from the
    metadata file.

    """
    Landsat_meta = open(filename, "r")  # Open metadata file
    for line in Landsat_meta:
        if re.match("(.*)SCENE_CENTER_TIME(.*)", line): # search in metadata for line SCENE_CENTER_TIME
            words = line.split()# make groups of words which are divided by an open space
            time_list = words[2].split(':', 2) # Take the second word of words and split the word which are divided by :
            if len(time_list[0])== 3:
                time_list[0]=time_list[0][1:3]
                time_list[2]=time_list[2][0:-1]
            hour = float(time_list[0]) # take the first word of time_list
            minutes = float(time_list[1]) + float(time_list[2][:-1]) / 60 # Take the second and third word of time_list and place :-1 to remove Z behined minutes
    Landsat_meta = open(filename, "r")  # Open metadata file
    for line in Landsat_meta:
        if re.match("(.*)DATE_ACQUIRED(.*)", line):
            words = line.split()
            DOY = time.strptime(words[2], "%Y-%m-%d").tm_yday
            year = time.strptime(words[2], "%Y-%m-%d").tm_year
    Landsat_meta = open(filename, "r")  # Open metadata file
    for line in Landsat_meta:
        if re.match("(.*)UTM_ZONE(.*)", line):
            words = line.split()
            UTM_Zone = int(words[2])
    Landsat_meta = open(filename, "r")  # Open metadata file
    for line in Landsat_meta:
        if re.match("(.*)SUN_ELEVATION(.*)", line):
            words = line.split()
            Sun_elevation = float(words[2])

    return year, DOY, hour, minutes, UTM_Zone, Sun_elevation

def Calc_vegt_para(NDVI,water_mask_temp):
    """
    Calculates the Fraction of PAR, Thermal infrared emissivity, Nitrogen, Vegetation Cover, LAI, b10_emissivity
    """
    # Fraction of PAR absorbed by the vegetation canopy (FPAR):
    FPAR = -0.161 + 1.257 * NDVI
    FPAR[NDVI < 0.125] = 0.0

    # Termal infrared emissivity
    tir_emis = 1.009 + 0.047 * np.log(NDVI)
    tir_emis[np.logical_or(water_mask_temp == 1.0, water_mask_temp == 2.0)] = 1.0
    tir_emis[np.logical_and(NDVI < 0.125, water_mask_temp == 0.0)] = 0.92

    # Vegetation Index - Regression model from Bagheri et al. (2013)
    VI = 38.764 * np.square(NDVI) - 24.605 * NDVI + 5.8103

    # Nitrogen computation
    Nitrogen = np.copy(VI)
    Nitrogen[VI <= 0.0] = 0.0
    Nitrogen[NDVI <= 0.0] = 0.0

    # Vegetation cover:
    vegt_cover = 1 - np.power((0.8 - NDVI)/(0.8 - 0.125), 0.7)
    vegt_cover[NDVI < 0.125] = 0.0
    vegt_cover[NDVI > 0.8] = 0.99

    # Leaf Area Index (LAI)
    LAI_1 = np.log(-(vegt_cover - 1)) / -0.45
    LAI_1[LAI_1 > 8] = 8.0
    LAI_2 = (9.519 * np.power(NDVI, 3) + 0.104 * np.power(NDVI, 2) +
             1.236 * NDVI - 0.257)

    LAI = (LAI_1 + LAI_2) / 2.0  # Average LAI
    LAI[LAI < 0.001] = 0.001

    b10_emissivity = np.where(LAI <= 3.0, 0.95 + 0.01 * LAI, 0.98)
    b10_emissivity[water_mask_temp != 0.0] = 1.0

    return(FPAR,tir_emis,Nitrogen,vegt_cover,LAI,b10_emissivity)

def Calc_NDVI(Reflect):
    """
    This function calculates and returns the Surface albedo, NDVI by using the refectance from the landsat image.
    """
    # Computation of Normalized Difference Vegetation Index (NDVI)
    NDVI = ((Reflect[:, :, 3] - Reflect[:, :, 2]) /
            (Reflect[:, :, 3] + Reflect[:, :, 2]))

    return(NDVI)

def Calc_Gradient(dataset,pixel_spacing):
    """
    This function calculates the slope and aspect of a DEM map.
    """
    # constants
    deg2rad = np.pi / 180.0  # Factor to transform from degree to rad
    rad2deg = 180.0 / np.pi  # Factor to transform from rad to degree

    # Calculate slope
    x, y = np.gradient(dataset, pixel_spacing, pixel_spacing)
    hypotenuse_array = np.hypot(x,y)
    slope = np.arctan(hypotenuse_array) * rad2deg
    #slope = np.arctan(np.sqrt(np.square(x/pixel_spacing) + np.square(y/pixel_spacing))) * rad2deg

    # calculate aspect
    aspect = np.arctan2(y/pixel_spacing, -x/pixel_spacing) * rad2deg
    aspect = 180 + aspect

    return(deg2rad,rad2deg,slope,aspect)


def Calc_surface_water_temp(Temp_inst,Landsat_nr,Lmax,Lmin,therm_data,b10_emissivity,k1_c,k2_c,eact,shape_lsc,water_mask_temp,Bands_thermal,Rp,tau_sky,surf_temp_offset,Image_Type):
    """
    Calculates the surface temperature and create a water mask
    """

    # Spectral radiance for termal
    if Landsat_nr == 8:
        if Bands_thermal == 1:
            k1 = k1_c[0]
            k2 = k2_c[0]
            L_lambda_b10 = (Lmax[-1] - Lmin[-1]) / (65535-1) * therm_data[:, :, 0] + Lmin[-1]

            # Get Temperature
            Surface_temp = Get_Thermal(L_lambda_b10,Rp,Temp_inst,tau_sky,b10_emissivity,k1,k2)

        elif Bands_thermal == 2:
            L_lambda_b10 = (Lmax[-2] - Lmin[-2]) / (65535-1) * therm_data[:, :, 0] + Lmin[-2]
            L_lambda_b11 = (Lmax[-1] - Lmin[-1]) / (65535-1) * therm_data[:, :, 1] + Lmin[-1]

            # Brightness temperature
            # From Band 10:
            Temp_TOA_10 = (k2_c[0] / np.log(k1_c[0] / L_lambda_b10 + 1.0))
            # From Band 11:
            Temp_TOA_11 = (k2_c[1] / np.log(k1_c[1] / L_lambda_b11 + 1.0))
            # Combined:
            Surface_temp = (Temp_TOA_10 + 1.378 * (Temp_TOA_10 - Temp_TOA_11) +
                           0.183 * np.power(Temp_TOA_10 - Temp_TOA_11, 2) - 0.268 +
                           (54.30 - 2.238 * eact) * (1 - b10_emissivity))

    elif Landsat_nr == 7:
        k1=666.09
        k2=1282.71
        L_lambda_b6 = (Lmax[-1] - Lmin[-1]) / (256-1) * therm_data[:, :, 0] + Lmin[-1]

        # Brightness temperature - From Band 6:
        Surface_temp = Get_Thermal(L_lambda_b6,Rp,Temp_inst,tau_sky,b10_emissivity,k1,k2)

    elif Landsat_nr == 5:
        k1=607.76
        k2=1260.56
        L_lambda_b6 = ((Lmax[-1] - Lmin[-1]) / (256-1) * therm_data[:, :, 0] +
                       Lmin[-1])

       # Brightness temperature - From Band 6:
        Surface_temp = Get_Thermal(L_lambda_b6,Rp,Temp_inst,tau_sky,b10_emissivity,k1,k2)

    # Surface temperature
    Surface_temp = Surface_temp.clip(230.0, 360.0)

    # Cloud mask:
    temp_water = np.zeros((shape_lsc[1], shape_lsc[0]))
    temp_water = np.copy(Surface_temp)
    temp_water[water_mask_temp == 0.0] = np.nan
    temp_water_sd = np.nanstd(temp_water)     # Standard deviation
    temp_water_mean = np.nanmean(temp_water)  # Mean
    print('Mean water temperature = ', '%0.3f (Kelvin)' % temp_water_mean)
    print('SD water temperature = ', '%0.3f (Kelvin)' % temp_water_sd)
    cloud_mask = np.zeros((shape_lsc[1], shape_lsc[0]))
    #cloud_mask[Surface_temp < np.minimum((temp_water_mean - 1.0 * temp_water_sd -
    #           surf_temp_offset),290)] = 1.0

    return(Surface_temp, cloud_mask)

def Get_Thermal(lambda_b10,Rp,Temp_inst,tau_sky,TIR_Emissivity,k1,k2):

    # Narrow band downward thermal radiation from clear sky, rsky (W/m2/sr/µm)
    rsky = (1.807E-10 * np.power(Temp_inst + 273.15, 4) * (1 - 0.26 *
            np.exp(-7.77E-4 * np.power((-Temp_inst), -2))))
    
    print('Rsky = ', '%0.3f (W/m2/sr/µm)' % np.nanmean(rsky))

    # Corrected thermal radiance from the surface, Wukelikc et al. (1989):
    correc_lambda_b10 = ((lambda_b10 - Rp) / tau_sky -
                               (1.0 - TIR_Emissivity) * rsky)
    
    # Brightness temperature - From Band 10:
    Temp_TOA = (k2 / np.log(TIR_Emissivity * k1 /
                       correc_lambda_b10 + 1.0))

    return(Temp_TOA)

def DEM_lat_lon(data_in):
    """
    This function retrieves information about the latitude and longitude of the
    DEM map.

    """
    if str(type(data_in)) == "<class 'str'>":
        g = gdal.Open(data_in)     # Open DEM
    elif str(type(data_in)) == "<class 'osgeo.gdal.Dataset'>":
        g = data_in
    else:
        raise RuntimeError("Dataformat not suported")
        
    geo_t = g.GetGeoTransform()     # Get the Geotransform vector:
    x_size = g.RasterXSize          # Raster xsize - Columns
    y_size = g.RasterYSize          # Raster ysize - Rows

    # create a longitude and a latitude array
    lon = np.zeros((y_size, x_size))
    lat = np.zeros((y_size, x_size))
    for col in np.arange(x_size):
        lon[:, col] = geo_t[0] + col * geo_t[1] + geo_t[1]/2
        # ULx + col*(E-W pixel spacing) + E-W pixel spacing
    for row in np.arange(y_size):
        lat[row, :] = geo_t[3] + row * geo_t[5] + geo_t[5]/2
        # ULy + row*(N-S pixel spacing) + N-S pixel spacing,
        # negative as we will be counting from the UL corner

    return(lat, lon)

def w_time(GMT,lon_proy, DOY):
    """
    This function computes the hour angle (radians) of an image given the
    local time, longitude, and day of the year.

    """
    nrow, ncol = lon_proy.shape

    # Difference of the local time (LT) from Greenwich Mean Time (GMT) (hours):
    delta_GTM = lon_proy[int(nrow/2), int(ncol/2)] * 24 / 360
    if np.isnan(delta_GTM) == True:
         delta_GTM = np.nanmean(lon_proy) * np.nanmean(lon_proy)  * 24 / 360

    # Local Standard Time Meridian (degrees):
    LSTM = 15 * delta_GTM

    # Ecuation of time (EoT, minutes):
    B = 360./365 * (DOY-81)  # (degrees)
    EoT = 9.87*np.sin(np.deg2rad(2*B))-7.53*np.cos(np.deg2rad(B))-1.5*np.sin(np.deg2rad(B))

    # Net Time Correction Factor (minutes) at the center of the image:
    TC = 4 * (lon_proy - LSTM) + EoT     # Difference in time over the longitude
    LST = GMT + delta_GTM + TC/60         # Local solar time (hours)
    HRA = 15 * (LST-12)                  # Hour angle HRA (degrees)
    w = np.deg2rad(HRA)                    # Hour angle HRA (radians)
    return w

def reproject_dataset(dataset, pixel_spacing, UTM_Zone, fit_extend = False):
    """
    A sample function to reproject and resample a GDAL dataset from within
    Python. The idea here is to reproject from one system to another, as well
    as to change the pixel size. The procedure is slightly long-winded, but
    goes like this:

    1. Set up the two Spatial Reference systems.
    2. Open the original dataset, and get the geotransform
    3. Calculate bounds of new geotransform by projecting the UL corners
    4. Calculate the number of pixels with the new projection & spacing
    5. Create an in-memory raster dataset
    6. Perform the projection
    """

    # 1) Open the dataset
    try:                               #!!! Doe deze met de type check
        g = gdal.Open(dataset)
    except:
        g = dataset
   
     # Define the EPSG code...
    EPSG_code = int(UTM_Zone)
    epsg_to = int(EPSG_code)

    # 2) Define the UK OSNG, see <http://spatialreference.org/ref/epsg/27700/>
    try:
        proj = g.GetProjection()
        Proj_in=proj.split('EPSG","')
        epsg_from=int((str(Proj_in[-1]).split(']')[0])[0:-1])
    except:
        epsg_from = int(4326)    # Get the Geotransform vector:
    geo_t = g.GetGeoTransform()

    # Vector components:
    # 0- The Upper Left easting coordinate (i.e., horizontal)
    # 1- The E-W pixel spacing
    # 2- The rotation (0 degrees if image is "North Up")
    # 3- The Upper left northing coordinate (i.e., vertical)
    # 4- The rotation (0 degrees)
    # 5- The N-S pixel spacing, negative as it is counted from the UL corner
    x_size = g.RasterXSize  # Raster xsize
    y_size = g.RasterYSize  # Raster ysize

    epsg_to = int(epsg_to)

    # 2) Define the UK OSNG, see <http://spatialreference.org/ref/epsg/27700/>
    osng = osr.SpatialReference()
    osng.ImportFromEPSG(epsg_to)
    wgs84 = osr.SpatialReference()
    wgs84.ImportFromEPSG(epsg_from)

    inProj = Proj(init='epsg:%d' %epsg_from)
    outProj = Proj(init='epsg:%d' %epsg_to)

    # Remove a part of image
    nrow_skip = 0 #round((0.06*y_size)/2)
    ncol_skip = 0 #round((0.06*x_size)/2)

    # Up to here, all  the projection have been defined, as well as a
    # transformation from the from to the to
    ulx, uly = transform(inProj,outProj,geo_t[0] + nrow_skip * geo_t[1], geo_t[3] + nrow_skip * geo_t[5])
    lrx, lry = transform(inProj,outProj,geo_t[0] + geo_t[1] * (x_size-ncol_skip),
                                        geo_t[3] + geo_t[5] * (y_size-nrow_skip))

    # See how using 27700 and WGS84 introduces a z-value!
    # Now, we create an in-memory raster
    mem_drv = gdal.GetDriverByName('MEM')

    if fit_extend == True:       
        ulx = np.ceil(ulx/pixel_spacing) * pixel_spacing + 0.5 * pixel_spacing
        uly = np.floor(uly/pixel_spacing) * pixel_spacing - 0.5 * pixel_spacing
        lrx = np.floor(lrx/pixel_spacing) * pixel_spacing - 0.5 * pixel_spacing
        lry = np.ceil(lry/pixel_spacing) * pixel_spacing + 0.5 * pixel_spacing

    else:       
        ulx = np.ceil(ulx/pixel_spacing) * pixel_spacing
        uly = np.floor(uly/pixel_spacing) * pixel_spacing
        lrx = np.floor(lrx/pixel_spacing) * pixel_spacing
        lry = np.ceil(lry/pixel_spacing) * pixel_spacing
 
    # The size of the raster is given the new projection and pixel spacing
    # Using the values we calculated above. Also, setting it to store one band
    # and to use Float32 data type.
    col = int((lrx - ulx)/pixel_spacing)
    rows = int((uly - lry)/pixel_spacing)

    # Re-define lr coordinates based on whole number or rows and columns
    (lrx, lry) = (ulx + col * pixel_spacing, uly -
                  rows * pixel_spacing)

    dest = mem_drv.Create('', col, rows, 1, gdal.GDT_Float32)

    if dest is None:
        print('input folder to large for memory, clip input map')

   # Calculate the new geotransform
    new_geo = (ulx, pixel_spacing, geo_t[2], uly,
               geo_t[4], - pixel_spacing)

    # Set the geotransform
    dest.SetGeoTransform(new_geo)
    dest.SetProjection(osng.ExportToWkt())

    # Perform the projection/resampling
    gdal.ReprojectImage(g, dest, wgs84.ExportToWkt(), osng.ExportToWkt(),gdal.GRA_CubicSpline)

    return dest, ulx, lry, lrx, uly, epsg_to

    
def Create_Example_UTM_File(filename_ex, pixel_spacing, proj):

    # Reproject DEM to fit LS scene
    dest, ulx, lry, lrx, uly, epsg_to = reproject_dataset(
            filename_ex, pixel_spacing, UTM_Zone = proj, fit_extend = True) 
    
    # Find example geotransform info
    Array = dest.GetRasterBand(1).ReadAsArray()
    geo = dest.GetGeoTransform()
    proj = dest.GetProjection()
    ncol = dest.RasterXSize        # Get the reprojected dem column size
    nrow = dest.RasterYSize        # Get the reprojected dem row size
    shape = [ncol, nrow]   
    
    return(dest, Array, geo, proj, ncol, nrow, shape)
    
def Calc_Coz_zn_dr(GMT_offset, DOY, hour_loc, minutes_loc, lon_proy, lat_proy, slope, aspect):
    """
    Calculates the extraterrestiral solar radiation by using the date, slope and aspect.
    """

    # Constants
    deg2rad = np.pi / 180.0  # Factor to transform from degree to rad
    Min_cos_zn = 0.1  # Min value for cos zenith angle
    Max_cos_zn = 1.0  # Max value for cos zenith angle
     
    try:
        GMT_time = float(hour_loc) - GMT_offset + float(minutes_loc)/60  # Local time (hours)
        Loc_time = float(hour_loc) + float(minutes_loc)/60  # Local time (hours)        
    except:
        GMT_time = np.float_(hour_loc) - GMT_offset + np.float_(minutes_loc)/60  # Local time (hours)
        Loc_time = np.float_(hour_loc) + np.float_(minutes_loc)/60  # Local time (hours)

    print('  Local Time: ', '%0.3f' % np.nanmean(Loc_time))
    print('  GMT Time: ', '%0.3f' % np.nanmean(GMT_time))    
    print('  Difference of local time (LT) from Greenwich (GMT): ', GMT_offset)

    # 1. Calculation of extraterrestrial solar radiation for slope and aspect
    # Computation of Hour Angle (HRA = w)
    B = 360./365 * (DOY-81)           # (degrees)
    # Computation of cos(theta), where theta is the solar incidence angle
    # relative to the normal to the land surface
    delta=np.arcsin(np.sin(23.45*deg2rad)*np.sin(np.deg2rad(B))) # Declination angle (radians)
    phi = lat_proy * deg2rad                                     # latitude of the pixel (radians)
    s = slope * deg2rad                                          # Surface slope (radians)
    gamma = (aspect-180) * deg2rad                               # Surface aspect angle (radians)
    w = w_time(GMT_time, lon_proy, DOY)                            # Hour angle (radians)
    a,b,c = Constants(delta,s,gamma,phi)
    cos_zn= AngleSlope(a,b,c,w)
    cos_zn = cos_zn.clip(Min_cos_zn, Max_cos_zn)

    print('Average Cos Zenith Angle: ', '%0.3f (Radians)' % np.nanmean(cos_zn))

    dr = 1 + 0.033 * np.cos(DOY*2*np.pi/365)  # Inverse relative distance Earth-Sun

    return(cos_zn, dr)    

def AngleSlope(a,b,c,w):
    '''
    Based on Richard G. Allen 2006
    Calculate the cos zenith angle by using the hour angle and constants
    '''
    angle = -a + b*np.cos(w) + c*np.sin(w)

    return(angle)

def Constants(delta,s,gamma,phi):
    '''
    Based on Richard G. Allen 2006 equation 11
    determines constants for calculating the exterrestial solar radiation
    '''
    a = np.sin(delta)*np.cos(phi)*np.sin(s)*np.cos(gamma) - np.sin(delta)*np.sin(phi)*np.cos(s)
    b = np.cos(delta)*np.cos(phi)*np.cos(s) + np.cos(delta)*np.sin(phi)*np.sin(s)*np.cos(gamma)
    c = np.cos(delta)*np.sin(s)*np.sin(gamma)

    return(a,b,c)

def Get_Time_Info(workbook, number):

   # Open the General input sheet
    ws = workbook['General_Input']

    # Extract the input and output folder, and Image type from the excel file
    input_folder = r"%s" %str(ws['B%d' %number].value)

    # Open the Landsat_Input sheet
    ws = workbook['Landsat_Input']

    # Extract Landsat name, number and amount of thermal bands from excel file
    Name_Landsat_Image = str(ws['B%d' %number].value)
    Landsat_nr = int(ws['C%d' %number].value)            # Type of Landsat (LS) image used (LS5, LS7, or LS8)

    # the path to the MTL file of landsat
    Landsat_meta_fileName = os.path.join(input_folder, '%s_MTL.txt' % Name_Landsat_Image)

    # read out the general info out of the MTL file
    year, DOY, hour, minutes, UTM_Zone, Sun_elevation = info_general_metadata(Landsat_meta_fileName) # call definition info_general_metadata

    return(year, DOY, hour, minutes, UTM_Zone, Sun_elevation, Landsat_nr)


def Get_LS_Para_Veg(workbook, number, Example_fileName, year, month, day, path_radiance, Apparent_atmosf_transm, cos_zn, dr, vegt_cover_NDVI_max):

    # Open the General input sheet
    ws = workbook['General_Input']

    # Extract the input and output folder, and Image type from the excel file
    input_folder = r"%s" %str(ws['B%d' %number].value)
    output_folder = r"%s" %str(ws['C%d' %number].value)

    ws = workbook['Additional_Input']

    # If all additional fields are filled in than do not open the datasets
    if ws['B%d' % number].value is None or ws['C%d' % number].value is None:

        print('-------------------- Open Landsat VIS -----------------------')

        # Open the Landsat_Input sheet
        ws = workbook['Landsat_Input']

        # Extract Landsat name, number and amount of thermal bands from excel file
        Name_Landsat_Image = str(ws['B%d' %number].value)
        Landsat_nr = int(ws['C%d' %number].value)            # Type of Landsat (LS) image used (LS5, LS7, or LS8)
                                                             # temperature: 1 = Band 6 for LS_5 & 7, Band 10 for LS_8 (optional)
        # Define bands used for each Landsat number
        if Landsat_nr == 5 or Landsat_nr == 7:
            Bands = np.array([1, 2, 3, 4, 5, 7, 6])
        elif Landsat_nr == 8:
           Bands = np.array([2, 3, 4, 5, 6, 7, 10, 11])
        else:
            print('Landsat image not supported, use Landsat 7 or 8')

        # Open MTL landsat and get the correction parameters
        Landsat_meta_fileName = os.path.join(input_folder, '%s_MTL.txt' %Name_Landsat_Image)
        Lmin, Lmax, k1_c, k2_c = info_band_metadata(Landsat_meta_fileName, Bands)
        print('Lmin= ', Lmin)
        print('Lmax= ', Lmax)
        print('k1= ', k1_c)
        print('k2= ', k2_c)

        sensor1 = 'LS%d' %Landsat_nr
        res1 = '30m'
        res2 = '30m'
        res3 = '30m'

        # Mean solar exo-atmospheric irradiance for each band (W/m2/microm)
        # for the different Landsat images (L5, L7, or L8)
        ESUN_L5 = np.array([1983, 1796, 1536, 1031, 220, 83.44])
        ESUN_L7 = np.array([1997, 1812, 1533, 1039, 230.8, 84.9])
        ESUN_L8 = np.array([1973.28, 1842.68, 1565.17, 963.69, 245, 82.106])

        # Open one band - To get the metadata of the landsat images only once (to get the extend)
        src_FileName = os.path.join(input_folder, '%s_B2.TIF' %Name_Landsat_Image)  # before 10!
        ls, band_data, ulx, uly, lrx, lry, x_size_ls, y_size_ls = Get_Extend_Landsat(src_FileName)
        print('Original LANDSAT Image - ')
        print('  Size :', x_size_ls, y_size_ls)
        print('  Upper Left corner x, y: ', ulx, ', ', uly)
        print('  Lower right corner x, y: ', lrx, ', ', lry)

        lsc = RC.reproject_dataset_example(src_FileName, Example_fileName)

        #	Get the extend of the remaining landsat file	after clipping based on the DEM file
        y_size_lsc = lsc.RasterYSize
        x_size_lsc = lsc.RasterXSize
        shape_lsc = [x_size_lsc, y_size_lsc]

        print('--- ')
        print('Cropped LANDSAT Image - ')
        print('  Size :', x_size_lsc, y_size_lsc)
        print('  Upper Left corner x, y: ', ulx, ', ',  uly)
        print('  Lower right corner x, y: ', lrx, ', ', lry)

        # if landsat 5 or 7 is used then first create a mask for removing the no data stripes
        if Landsat_nr == 5 or Landsat_nr == 7:
            src_FileName = os.path.join(input_folder, '%s_B6.TIF' % (Name_Landsat_Image)) #open smallest band
            if not os.path.exists(src_FileName):
                src_FileName = os.path.join(input_folder, '%s_B6_VCID_2.TIF' % (Name_Landsat_Image))
            src_FileName_2 = os.path.join(input_folder, '%s_B1.TIF' % (Name_Landsat_Image)) #open smallest band
            src_FileName_3 = os.path.join(input_folder, '%s_B3.TIF' % (Name_Landsat_Image)) #open smallest band
            src_FileName_4 = os.path.join(input_folder, '%s_B4.TIF' % (Name_Landsat_Image)) #open smallest band
            src_FileName_5 = os.path.join(input_folder, '%s_B7.TIF' % (Name_Landsat_Image)) #open smallest band
            src_FileName_6 = os.path.join(input_folder, '%s_B2.TIF' % (Name_Landsat_Image)) #open smallest band
            src_FileName_7 = os.path.join(input_folder, '%s_B5.TIF' % (Name_Landsat_Image)) #open smallest band
            ls_data=Open_landsat(src_FileName,Example_fileName)
            ls_data_2=Open_landsat(src_FileName_2,Example_fileName)
            ls_data_3=Open_landsat(src_FileName_3,Example_fileName)
            ls_data_4=Open_landsat(src_FileName_4,Example_fileName)
            ls_data_5=Open_landsat(src_FileName_5,Example_fileName)
            ls_data_6=Open_landsat(src_FileName_6,Example_fileName)
            ls_data_7=Open_landsat(src_FileName_7,Example_fileName)

            # create and save the landsat mask for all images based on band 11 (smallest map)
            QC_Map=np.zeros((shape_lsc[1], shape_lsc[0]))
            QC_Map=np.where(np.logical_or.reduce((ls_data==0,ls_data_2==0,ls_data_3==0,ls_data_4==0,ls_data_5==0,ls_data_6==0,ls_data_7==0)),1,0)

        # If landsat 8 then use landsat band 10 and 11
        elif Landsat_nr == 8:
             src_FileName_11 = os.path.join(input_folder, '%s_B11.TIF' % (Name_Landsat_Image)) #open smallest band
             ls_data_11=Open_landsat(src_FileName_11,Example_fileName)

             src_FileName_10 = os.path.join(input_folder, '%s_B10.TIF' % (Name_Landsat_Image)) #open smallest band
             ls_data_10=Open_landsat(src_FileName_10, Example_fileName)

             # create and save the landsat mask for all images based on band 10 and 11
             QC_Map=np.zeros((shape_lsc[1], shape_lsc[0]))
             QC_Map=np.where(np.logical_or(ls_data_11==0, ls_data_10==0),1,0)

        else:
            print('Landsat image not supported, use Landsat 7 or 8')

        # Open data of the landsat mask
        ls_data=Open_landsat(src_FileName, Example_fileName)

        # Create 3D array to store Spectral radiance and Reflectivity for each band
        Reflect, Spec_Rad = Landsat_Reflect(Bands, input_folder, Name_Landsat_Image, shape_lsc, QC_Map, Lmax, Lmin, ESUN_L5, ESUN_L7, ESUN_L8, cos_zn, dr, Landsat_nr, Example_fileName)

        # save spectral data
        for i in range(0,6):
            spec_ref_fileName = os.path.join(output_folder, 'Output_radiation_balance','%s_spectral_reflectance_B%s_%s_%s%02d%02d.tif' %(sensor1, Bands[i], res3, year, month, day))
            sebal.save_GeoTiff_proy(lsc, Reflect[:, :, i], spec_ref_fileName, shape_lsc, nband=1)

    else:
        # Get General information example file
        lsc = gdal.Open(Example_fileName)
        nrow = lsc.RasterYSize
        ncol = lsc.RasterXSize
        shape_lsc = [ncol, nrow]

    ######################### Calculate Vegetation Parameters Based on VIS data #####################################

    # Open the Additional input excel sheet
    ws = workbook['Additional_Input']

    # Check NDVI and Calculate NDVI
    try:
        if (ws['B%d' % number].value) is not None:

            # Output folder NDVI
            ndvi_fileName_user = os.path.join(output_folder, 'Output_vegetation', 'User_NDVI_%s_%s%02d%02d.tif' %(res3, year, month, day))
            NDVI=sebal.Reshape_Reproject_Input_data(r'%s' %str(ws['B%d' % number].value),ndvi_fileName_user,Example_fileName)

            water_mask_temp = np.zeros((shape_lsc[1], shape_lsc[0]))
            water_mask_temp[NDVI < 0.0] = 1.0
            sebal.save_GeoTiff_proy(lsc, NDVI, ndvi_fileName_user, shape_lsc, nband=1)

        else:
            # use the Landsat reflectance to calculate the surface albede, NDVI
            NDVI = Calc_NDVI(Reflect)

            # Calculate temporal water mask
            water_mask_temp=sebal.Water_Mask(shape_lsc,Reflect)

    except:
        assert "Please check the NDVI input path"

    # Check Water Mask and replace if it is filled in the additianal data sheet
    try:
        if (ws['E%d' % number].value) is not None:

            # Overwrite the Water mask and change the output name
            water_mask_temp_fileName = os.path.join(output_folder, 'Output_soil_moisture', 'User_Water_mask_temporary_%s_%s%02d%02d.tif' %(res2, year, month, day))
            water_mask_temp = sebal.Reshape_Reproject_Input_data(r'%s' %str(ws['E%d' % number].value), water_mask_temp_fileName, Example_fileName)
            sebal.save_GeoTiff_proy(lsc, water_mask_temp, water_mask_temp_fileName, shape_lsc, nband=1)

    except:
        assert "Please check the Water Mask input path"

    # Check Surface albedo
    try:
        if (ws['C%d' % number].value) is not None:

            # Output folder surface albedo
            surface_albedo_fileName = os.path.join(output_folder, 'Output_vegetation','User_surface_albedo_%s_%s%02d%02d.tif' %(res2, year, month, day))
            Surf_albedo=sebal.Reshape_Reproject_Input_data(r'%s' %str(ws['C%d' % number].value),surface_albedo_fileName,Example_fileName)
            sebal.save_GeoTiff_proy(lsc, Surf_albedo, surface_albedo_fileName, shape_lsc, nband=1)

        else:

            # use the Landsat reflectance to calculate the surface albede, NDVI
            Surf_albedo = sebal.Calc_albedo(Reflect, path_radiance, Apparent_atmosf_transm, Landsat_nr)

    except:
          assert "Please check the Albedo input path"

    reflectance_B12 = Reflect[:,:,5]

    # calculate vegetation properties
    FPAR,tir_emis,Nitrogen,vegt_cover,LAI,b10_emissivity=Calc_vegt_para(NDVI, water_mask_temp, vegt_cover_NDVI_max)

    print('Average NDVI = %s' %np.nanmean(NDVI))
    print('Average Surface Albedo = %s' %np.nanmean(Surf_albedo))
    print('Average LAI = %s' %np.nanmean(LAI))
    print('Average Vegetation Cover = %s' %np.nanmean(vegt_cover))
    print('Average FPAR = %s' %np.nanmean(FPAR))

    if not "QC_Map" in locals():
       QC_Map=np.zeros((shape_lsc[1], shape_lsc[0])) 

    return(Surf_albedo, NDVI, LAI, vegt_cover, FPAR, Nitrogen, tir_emis, b10_emissivity, water_mask_temp, QC_Map, reflectance_B12)

def Get_LS_Para_Thermal(workbook, number, Example_fileName, year, month, day, water_mask_temp, b10_emissivity, Temp_inst, Rp, tau_sky, surf_temp_offset, Thermal_Sharpening_not_needed, DEM_fileName, UTM_Zone, eact_inst, QC_Map):

    # import IrriEngine.pySEBAL.pySEBAL_code as SEBAL

    # Open the General input sheet
    ws = workbook['General_Input']

    # Extract the input and output folder, and Image type from the excel file
    input_folder = r"%s" %str(ws['B%d' %number].value)
    output_folder = r"%s" %str(ws['C%d' %number].value)
    Image_Type = 1

    # Open General information example file
    lsc = gdal.Open(Example_fileName)
    nrow = lsc.RasterYSize
    ncol = lsc.RasterXSize
    shape_lsc = [ncol, nrow]

    # Open the Landsat_Input sheet
    ws = workbook['Landsat_Input']

    # Extract Landsat name, number and amount of thermal bands from excel file
    Name_Landsat_Image = str(ws['B%d' %number].value)
    Bands_thermal = int(ws['D%d' %number].value)
    Landsat_nr = int(ws['C%d' %number].value)            # Type of Landsat (LS) image used (LS5, LS7, or LS8)
                                                         # temperature: 1 = Band 6 for LS_5 & 7, Band 10 for LS_8 (optional)
    # Define bands used for each Landsat number
    if Landsat_nr == 5 or Landsat_nr == 7:
        Bands = np.array([1, 2, 3, 4, 5, 7, 6])
    elif Landsat_nr == 8:
        Bands = np.array([2, 3, 4, 5, 6, 7, 10, 11])
    else:
        print('Landsat image not supported, use Landsat 7 or 8')

    # Open MTL landsat and get the correction parameters
    Landsat_meta_fileName = os.path.join(input_folder, '%s_MTL.txt' %Name_Landsat_Image)
    Lmin, Lmax, k1_c, k2_c = info_band_metadata(Landsat_meta_fileName, Bands)

    sensor1 = 'LS%d' %Landsat_nr
    sensor2 = 'LS%d' %Landsat_nr
    res1 = '30m'
    res2 = '30m'
    res3 = '30m'

    # Open the Landsat_Input sheet
    ws = workbook['Additional_Input']

    # If all additional fields are filled in than do not open the datasets
    if ws['D%d' % number].value is None:

        # Define bands used for each Landsat number
        if Landsat_nr == 5 or Landsat_nr == 7:
            Bands = np.array([1, 2, 3, 4, 5, 7, 6])
        elif Landsat_nr == 8:
           Bands = np.array([2, 3, 4, 5, 6, 7, 10, 11])
        else:
            print('Landsat image not supported, use Landsat 7 or 8')

        print('...................... Open Landsat Thermal ........................')

        # Check if a surface temperature dataset is defined. If so use this one instead of the Landsat, otherwise Landsat
        therm_data = Landsat_therm_data(Bands, input_folder, Name_Landsat_Image, shape_lsc, QC_Map, Example_fileName)

        # Create Cloud mask if BQA map is available (newer version Landsat images)
        BQA_LS_Available = 0
        if os.path.exists(os.path.join(input_folder, '%s_BQA.TIF' %Name_Landsat_Image)):
            src_FileName_BQA = os.path.join(input_folder, '%s_BQA.TIF' %Name_Landsat_Image)
            ls_data_BQA = Open_landsat(src_FileName_BQA, Example_fileName)
            if Landsat_nr == 8:
                Cloud_Treshold = 3700  #2720
            if Landsat_nr == 5 or Landsat_nr == 7:
                Cloud_Treshold = 700
            QC_mask_Cloud = np.copy(ls_data_BQA)
            QC_mask_Cloud[ls_data_BQA<Cloud_Treshold] = 0
            QC_mask_Cloud[ls_data_BQA>=Cloud_Treshold] = 1
            BQA_LS_Available = 1

        # Calculate surface temperature and create a cloud mask
        Surface_temp, cloud_mask_temp = Calc_surface_water_temp(Temp_inst, Landsat_nr, Lmax, Lmin, therm_data, b10_emissivity, k1_c, k2_c, eact_inst, shape_lsc, water_mask_temp, Bands_thermal, Rp, tau_sky, surf_temp_offset, Image_Type)

        # Replace clouds mask calculated by SEBAL by the official BQA file if this exists
        if BQA_LS_Available == 1:
            cloud_mask_temp = QC_mask_Cloud

        Surface_temp[cloud_mask_temp == 1] = np.nan
        print('Mean Surface Temperature = %s Kelvin' %np.nanmean(Surface_temp))

    else:
        try:
            # Output folder surface temperature
            surf_temp_fileName = os.path.join(output_folder, 'Output_vegetation','User_surface_temp_%s_%s%02d%02d.tif' %(res2, year, month, day))
            Surface_temp = sebal.Reshape_Reproject_Input_data(r'%s' %str(ws['D%d' % number].value),surf_temp_fileName,Example_fileName)
            cloud_mask_temp = np.zeros([int(np.shape(Surface_temp)[0]),int(np.shape(Surface_temp)[1])])
            Thermal_Sharpening_not_needed = 0

        except:
            assert "Please check the surface temperature input path"

    return(Surface_temp, cloud_mask_temp, Thermal_Sharpening_not_needed)

#------missing functions
# def Reshape_Reproject_Input_data():
#     return

# def Calc_surface_water_temp():
#     return 

# def save_GeoTiff_proy():
#     return

# def Water_Mask():
#     return

# def Calc_albedo():
#     return

#------------------------------------------------------------------------------
def info_band_metadata(filename, Bands):
    """
    This function retrieves Landsat band information (minimum and maximum
    radiance) from the metadata file.

    """
    Lmin = np.zeros(len(Bands))  # Minimum band radiance, for each band
    Lmax = np.zeros(len(Bands))  # Maximum band radiance, for each band
    k1_const = np.zeros(len(Bands)-6)  # TIRS_Thermal constant k1 ######
    k2_const = np.zeros(len(Bands)-6)  # TIRS_Thermal constant k2 ######
    for band in Bands:
        Landsat_meta = open(filename, "r")  # Open metadata file
        for line in Landsat_meta:
            if re.match("(.*)RADIANCE_MINIMUM_BAND_%1d(.*)" % band, line):
                words = line.split()
                value = float(words[2])
                Lmin[np.where(Bands == band)[0][0]] = value
            if re.match("(.*)RADIANCE_MAXIMUM_BAND_%1d(.*)" % band, line):
                words = line.split()
                value = float(words[2])
                Lmax[np.where(Bands == band)[0][0]] = value
            if re.match("(.*)K1_CONSTANT_BAND_%1d(.*)" % band, line):  # #####
                words = line.split()
                value = float(words[2])
                k1_const[np.where(Bands == band)[0][0]-6] = value
            if re.match("(.*)K2_CONSTANT_BAND_%1d(.*)" % band, line):  # #####
                words = line.split()
                value = float(words[2])
                k2_const[np.where(Bands == band)[0][0]-6] = value
    return Lmin, Lmax, k1_const, k2_const

#------------------------------------------------------------------------------
def Get_Extend_Landsat(src_FileName):
    """
    This function gets the extend of the landsat image
    """
    ls = gdal.Open(src_FileName)       # Open Landsat image
    geo_t_ls = ls.GetGeoTransform()    # Get the Geotransform vector
    x_size_ls = ls.RasterXSize         # Raster xsize - Columns
    y_size_ls = ls.RasterYSize         # Raster ysize - Rows
    (ulx, uly) = geo_t_ls[0], geo_t_ls[3]
    (lrx, lry) = (geo_t_ls[0] + geo_t_ls[1] * x_size_ls,
                  geo_t_ls[3] + geo_t_ls[5] * y_size_ls)
    band_data = ls.GetRasterBand(1)

    return(ls,band_data,ulx,uly,lrx,lry,x_size_ls,y_size_ls)

#------------------------------------------------------------------------------
def Landsat_Reflect(Bands,input_folder,Name_Landsat_Image,shape_lsc,QC_Map,Lmax,Lmin,ESUN_L5,ESUN_L7,ESUN_L8,cos_zn,dr,Landsat_nr, proyDEM_fileName):
    """
    This function calculates and returns the reflectance and spectral radiation from the landsat image.
    """

    Spec_Rad = np.zeros((shape_lsc[1], shape_lsc[0], 7))
    Reflect = np.zeros((shape_lsc[1], shape_lsc[0], 7))
    for band in Bands[:-(len(Bands)-6)]:
        # Open original Landsat image for the band number
        src_FileName = os.path.join(input_folder, '%s_B%1d.TIF'
                                    % (Name_Landsat_Image, band))

        ls_data=Open_landsat(src_FileName, proyDEM_fileName)
        ls_data = ls_data * QC_Map
        # stats = band_data.GetStatistics(0, 1)

        index = np.where(Bands[:-(len(Bands)-6)] == band)[0][0]
        if Landsat_nr == 8:
            # Spectral radiance for each band:
            L_lambda = Landsat_L_lambda(Lmin, Lmax, ls_data, index, Landsat_nr)
            # Reflectivity for each band:
            rho_lambda = Landsat_rho_lambda(L_lambda, ESUN_L8, index, cos_zn, dr)
        elif Landsat_nr == 7:
            # Spectral radiance for each band:
            L_lambda=Landsat_L_lambda(Lmin, Lmax, ls_data, index, Landsat_nr)
            # Reflectivity for each band:
            rho_lambda = Landsat_rho_lambda(L_lambda, ESUN_L7, index, cos_zn, dr)
        elif Landsat_nr == 5:
            # Spectral radiance for each band:
            L_lambda=Landsat_L_lambda(Lmin, Lmax, ls_data, index, Landsat_nr)
            # Reflectivity for each band:
            rho_lambda =Landsat_rho_lambda(L_lambda, ESUN_L5, index, cos_zn, dr)
        else:
            print('Landsat image not supported, use Landsat 5, 7 or 8')

        Spec_Rad[:, :, index] = L_lambda
        Reflect[:, :, index] = rho_lambda
    Reflect = Reflect.clip(0.0, 1.0)
    return(Reflect,Spec_Rad)


#------------------------------------------------------------------------------
def Landsat_L_lambda(Lmin,Lmax,ls_data,index,Landsat_nr):
    """
    Calculates the lambda from landsat
    """
    if Landsat_nr==8:
        L_lambda = ((Lmax[index] - Lmin[index]) / (65535 - 1) * ls_data + Lmin[index])
    elif Landsat_nr == 5 or Landsat_nr ==7:
        L_lambda = (Lmax[index] - Lmin[index]) / 255 * ls_data + Lmin[index]
    return(L_lambda)


#------------------------------------------------------------------------------
def Landsat_rho_lambda(L_lambda,ESUN,index,cos_zn,dr):
    """
    Calculates the rho from landsat
    """
    rho_lambda = np.pi * L_lambda / (ESUN[index] * cos_zn * dr)
    return(rho_lambda)


#------------------------------------------------------------------------------
def Landsat_therm_data(Bands, input_folder, Name_Landsat_Image, shape_lsc, QC_Map, proyDEM_fileName):
    """
    This function calculates and returns the thermal data from the landsat image.
    """

    therm_data = np.zeros((shape_lsc[1], shape_lsc[0], len(Bands)-6))
    for band in Bands[-(len(Bands)-6):]:
        
        # Open original Landsat image for the band number
        src_FileName = os.path.join(input_folder, '%s_B%1d.TIF'
                                    % (Name_Landsat_Image, band))
        if not os.path.exists(src_FileName):
             src_FileName = os.path.join(input_folder, '%s_B%1d_VCID_2.TIF'
                                    % (Name_Landsat_Image, band))

        ls_data = Open_landsat(src_FileName, proyDEM_fileName)
        ls_data = ls_data * QC_Map
        index = np.where(Bands[:] == band)[0][0] - 6
        therm_data[:, :, index] = ls_data

    return(therm_data)

#------------------------------------------------------------------------------
def Open_landsat(src_FileName, proyDEM_fileName):
    """
    This function opens a landsat image and returns the data array of a specific landsat band.
    """

    # crop band to the DEM extent
    ls = RC.reproject_dataset_example(src_FileName, proyDEM_fileName)

    # Open the cropped Landsat image for the band number
    ls_data = ls.GetRasterBand(1).ReadAsArray()
    return(ls_data)

#%%

if __name__ == "__main__":
    LS_input_folder = r"/Volumes/Data/RS_scenes/india_case/C1_L1"
    output_folder = r"/Volumes/Data/RS_scenes/india_case/C1_L1_output"

    # Find dates
    os.chdir(LS_input_folder)
    files_LS = glob.glob("*.tar.gz")
    dates = [datetime.datetime.strptime(k.split("_")[3], "%Y%m%d") for k in files_LS]

    for date in dates:

        date_str = date.strftime("%Y%m%d")
        os.chdir(LS_input_folder)
        found = glob.glob("L*_*_%s_*_*_*.tar.gz" %date_str)
        
        if len(found) > 0:
            
            for found_one in found:

                extract_folder = os.path.join(LS_input_folder, found_one.replace(".tar.gz",""))
                if not os.path.exists(extract_folder):
                    os.makedirs(extract_folder)

                os.chdir(extract_folder)
                
                LS_file = os.path.splitext(os.path.splitext(found_one)[0])[0] +"_MTL.txt"
                Startdate_IrriEngine = datetime.datetime.strptime(LS_file.split("_")[3], "%Y%m%d")
                Startdate_IrriEngine = Startdate_IrriEngine.strftime("%Y-%m-%d")
                    
                file_LS_Cloud = os.path.join(output_folder, "%s" %LS_file.split("_")[2], "%s"  %LS_file.split("_")[3], "LS_CLOUD_%s.tif" %("_".join(LS_file.split("_")[0:4])))
                Tile_name = LS_file.split("_")[2]
                
                if not os.path.exists(file_LS_Cloud):
        
                    print("Process %s" %found_one)

                    LS_B1_filename = LS_file.replace("_MTL.txt", "_B1.tif")

                    if not os.path.exists(LS_B1_filename):
                        # Extract zip file
                        DC.Extract_Data_tar_gz(os.path.join(LS_input_folder,found_one), extract_folder)
                
                    # Get LS_filename
                    print(found_one)
                    Landsat_nr = int(found_one[3])
                    
                    # read out the general info out of the MTL file
                    year, DOY, hour, minutes, UTM_Zone, Sun_elevation = info_general_metadata(LS_file)     
                    UTM_Zone = int("326%02d" %UTM_Zone)#!!! nu alleen voor noord
                    folder_LS_RAW = os.path.join(LS_input_folder, "RAW")  
                    folder_LS_RAW_DEM = os.path.join(folder_LS_RAW, "%s" %(Tile_name))         
                    filename_DEM = os.path.join(folder_LS_RAW_DEM, "HydroSHED", "DEM", "DEM_HydroShed_m_3s.tif")
                        
                    if not os.path.exists(filename_DEM):
                        
                        if not os.path.exists(folder_LS_RAW_DEM):
                            os.makedirs(folder_LS_RAW_DEM)
                        
                        # Get extend in degrees of tile
                        dest = gdal.Open(os.path.join(extract_folder,LS_B1_filename))
                        geo = dest.GetGeoTransform()
                        epsg_from = Proj(init="epsg:%s" %UTM_Zone) 
                        epsg_to = Proj(init="epsg:4326")
                        x1 = geo[0]
                        x2 = geo[0] + dest.RasterXSize*geo[1]
                        y1 = geo[3]
                        y2 = geo[3] + dest.RasterYSize * geo[5]
                        x1_deg, y1_deg = transform(epsg_from, epsg_to, x1, y1)
                        x2_deg, y2_deg = transform(epsg_from, epsg_to, x2, y2)
                        
                        watertools.Collect.DEM.HydroSHED(folder_LS_RAW_DEM, [y2_deg - 0.1, y1_deg + 0.1], [x1_deg - 0.1, x2_deg + 0.1])
                        
                    else:
                            # Get extend in degrees of tile
                        dest = gdal.Open(os.path.join(extract_folder,LS_B1_filename))
                        geo = dest.GetGeoTransform()
                        epsg_from = Proj(init="epsg:%s" %UTM_Zone) 
                        epsg_to = Proj(init="epsg:4326")
                        x1 = geo[0]
                        x2 = geo[0] + dest.RasterXSize*geo[1]
                        y1 = geo[3]
                        y2 = geo[3] + dest.RasterYSize * geo[5]
                        x1_deg, y1_deg = transform(epsg_from, epsg_to, x1, y1)
                        x2_deg, y2_deg = transform(epsg_from, epsg_to, x2, y2)
                        
                    lon_ave = (x1_deg + x2_deg)/2  
                        
                    GMT_offset = round(lon_ave * 24 / 360)
                    
                    # Find Time to take
                    time_wheater_data = np.linspace(0,21,8) + 1.5

                    # Collect required METEO data
                    Time_Step = np.argwhere(time_wheater_data == Find_nearest_time(time_wheater_data, hour + minutes/60))[0][0]
                    hour_take = Time_Step * 3
                    
                    temp_format_inst = os.path.join(folder_LS_RAW, "Weather_Data", "Model", "GLDAS", "three_hourly", "tair_f_inst", "Tair_GLDAS-NOAH_C_3hour_{yyyy}.{mm:02d}.{dd:02d}_{HH:02d}00.tif")
                    RH_format_inst = os.path.join(folder_LS_RAW, "Weather_Data", "Model", "GLDAS", "three_hourly", "rh_f_inst", "Hum_GLDAS-NOAH_percentage_3hour_{yyyy}.{mm:02d}.{dd:02d}_{HH:02d}00.tif")
                
                    if not os.path.exists(temp_format_inst.format(yyyy=date.year, mm=date.month, dd=date.day, HH=hour_take)):
                        watertools.Collect.GLDAS.three_hourly(folder_LS_RAW, ['tair_f_inst'], date, date, [y2_deg - 0.3, y1_deg + 0.3], [x1_deg - 0.3, x2_deg + 0.3], Periods = [int(Time_Step+1)])
                                        
                    if not os.path.exists(RH_format_inst.format(yyyy=date.year, mm=date.month, dd=date.day, HH=hour_take)):          
                            watertools.Collect.GLDAS.three_hourly(folder_LS_RAW, ['psurf_f_inst', 'qair_f_inst'], date, date, [y2_deg - 0.3, y1_deg + 0.3], [x1_deg - 0.3, x2_deg + 0.3], Periods = [int(Time_Step+1)])
                            input_format_psurf = os.path.join(folder_LS_RAW, "Weather_Data", "Model", "GLDAS", "three_hourly", "psurf_f_inst", "P_GLDAS-NOAH_kpa_3hour_{yyyy}.{mm:02d}.{dd:02d}_{HH:02d}00.tif")
                            input_format_qair = os.path.join(folder_LS_RAW, "Weather_Data", "Model", "GLDAS", "three_hourly", "qair_f_inst", "Hum_GLDAS-NOAH_kg-kg_3hour_{yyyy}.{mm:02d}.{dd:02d}_{HH:02d}00.tif")
                            watertools.Products.RH.Use_Meteo.Calc_Humidity(temp_format_inst,
                                                                input_format_psurf,
                                                                input_format_qair,
                                                                RH_format_inst,
                                                                date, date+pd.DateOffset(days=1), freq = "3H")                
                        
                    # Define output folders
                    folder_LS_NDVI = os.path.join(output_folder, "%s" %(Tile_name), "NDVI")
                    folder_LS_Albedo = os.path.join(output_folder, "%s" %(Tile_name), "Albedo")
                    folder_LS_VSDI = os.path.join(output_folder, "%s" %(Tile_name), "VSDI")
                    folder_LS_NDII = os.path.join(output_folder, "%s" %(Tile_name), "NDII")
                    folder_LS_B12 = os.path.join(output_folder, "%s" %(Tile_name), "B12")
                    folder_LS_LST = os.path.join(output_folder, "%s" %(Tile_name), "LST")
                    folder_LS_Cloud = os.path.join(output_folder, "%s" %(Tile_name), "CLOUD")
                    
                    # Save output folders
                    FOLDERS = [extract_folder, folder_LS_RAW, folder_LS_NDVI, folder_LS_Albedo, folder_LS_VSDI, folder_LS_NDII, folder_LS_B12, folder_LS_Cloud, folder_LS_LST]

                    for folder in FOLDERS:
                        if not os.path.exists(folder) and folder != '':
                            print("create input folder %s" %folder)
                            os.makedirs(folder)
                            
                    Process_LS_Image(LS_file, FOLDERS, filename_DEM, UTM_Zone, Startdate_IrriEngine, GMT_offset)

                    shutil.rmtree(extract_folder)

#%%



# -*- coding: utf-8 -*-
"""
Authors: Laust Færch
Module: Collect/PROBAV

Description:
This module downloads Proba-V NDVI data from
https://www.vito-eodata.be/PDF/datapool/.
You need to register as a user (which is free) to use this module

"""
# from inspect import getsourcefile
# import sys
# from pathlib import Path

# probav_init_path = getsourcefile(lambda:0)
# probav_module_path = Path(probav_init_path).parent

# vito_download_path = Path.joinpath(probav_module_path, "vito-download")
# itsybitsy_path = Path.joinpath(probav_module_path, "itsybitsy")

# # Add paths for Gitmodules to sys.path
# sys.path.append(vito_download_path.as_posix())
# sys.path.append(itsybitsy_path.as_posix())

# # print(sys.path)

from .PROBAV_S5 import main as PROBAV_S5

__all__ = ["PROBAV_S5"]

__version__ = '0.1'



#%%
import os
from osgeo import gdal
import numpy as np
import datetime
import warnings
import pywapor.et_look_dev as ETLook_dev
import pywapor.et_look_v2 as ETLook_v2
import pywapor.general.outputs as out
import pywapor.general.variables as vars
import pywapor.general.processing_functions as PF
import copy
import xarray as xr

def get_geoinfo(example_filepath):
    ds = gdal.Open(example_filepath)
    geo_ex = ds.GetGeoTransform()
    proj_ex = ds.GetProjection()
    xsize = ds.RasterXSize
    ysize = ds.RasterYSize
    dlat, dlon = PF.calc_dlat_dlon(geo_ex, xsize, ysize)
    dem_resolution = (np.nanmean(dlon) + np.nanmean(dlat))/2
    return dem_resolution, geo_ex, proj_ex

def open_array(key, value, date, folders):
    fp = create_fp(key, value, date, folders)
    if os.path.exists(fp):
        # array = create_array(fp)
        ds = gdal.Open(fp)
        band = ds.GetRasterBand(1)
        ndv = band.GetNoDataValue()
        array = band.ReadAsArray()
        array[array == ndv] = np.nan
        return array
    else:
        # print("'{0}' not found.".format(key))
        return None

def create_fp(key, value, date, folders):
    if value["time"] == "daily":
        date = date.strftime("_%Y%m%d")
    elif value["time"] == "static":
        date = ""
    elif value["time"] == "yearly":
        date = date.strftime("_%Y")
    else:
        print("ERROR: invalid value")

    fn = "{0}{1}.tif".format(key, date)
    fp = os.path.join(folders[value["time"]], fn)

    return fp

def main(project_folder, date, level = "level_1", et_look_version = "v2", 
            output = None, input_data = None):

    # Version
    if et_look_version == "v2":
        ETLook = ETLook_v2
        print("--> Running ETLook_v2")
    elif et_look_version == "dev":
        ETLook = ETLook_dev
        print("--> Running ETLOOK_dev")
    c = ETLook.constants

    # Date
    if isinstance(date, str):
        date = datetime.datetime.strptime(date, "%Y-%m-%d")
    date_str = date.strftime("%Y%m%d")

    # Disable Warnings
    warnings.filterwarnings('ignore')

    # Folders
    input_folder_date = os.path.join(project_folder, level, date_str)
    input_folder_static = os.path.join(project_folder, level, "static")
    folders = {"daily": input_folder_date,
               "static": input_folder_static,
               "yearly": input_folder_static,}
    output_folder_date = os.path.join(project_folder, f"out_{level}", date_str)
    if not os.path.exists(output_folder_date) and isinstance(input_data, type(None)):
        os.makedirs(output_folder_date)

    if isinstance(output, type(None)):
        out_final = out.outputs
    elif output == "all":
        out_final = {k: True for k in out.outputs.keys()}
    elif isinstance(output, dict):
        for key, value in output.items():
            out.outputs[key] = value
            out_final = out.outputs

    for key, value in vars.outputs.items():
        value["output"] = out_final[key]
        value["file_path"] = os.path.join(output_folder_date, 
        "{0}_{1}.tif".format(value["file_name"], date_str))

    # Inputs
    
    id = {v["array_name"]: open_array(k, v, date, folders) for k, v in vars.inputs.items()}
    if not isinstance(input_data, type(None)):
        id = {**id, **input_data}

    spatial_or_constant = ["lw_offset", "lw_slope", "r0_bare", "r0_full", 
                            "rn_offset", "rn_slope", "t_opt", "vpd_slope"]

    # Logging
    for name, value in id.items():
        if name == "se_root" and isinstance(value, type(None)):
            print("----> se_root will be calculated from instantaneous data.")
        if name == "t_air_24" and isinstance(value, type(None)):
            print("----> t_air_24 will be calculated from t_air_min_24 and t_air_max_24.")
        if name == "ra_24" and isinstance(value, type(None)):
            print("----> ra_24 will be calculated from trans_24.")
        if name == "trans_24" and isinstance(value, type(None)):
            print("----> trans_24 will be calculated from ra_24.")
        if name == "z_oro" and isinstance(value, type(None)):
            print("----> z_oro will be calculated from slope.")
        if name == "u_24" and isinstance(value, type(None)):
            print("----> u_24 will be calculated from u2m_24 and v2m_24.")
        if name in spatial_or_constant and isinstance(value, type(None)):
            print(f"----> {name} will be constant at {getattr(c, name):.4f}.")
        if name in spatial_or_constant and np.isscalar(value):
            print(f"----> {name} will be constant at {value:.4f}.")

    # Resolution
    example_filepath = create_fp("r0", vars.inputs["r0"], date, folders)
    geoinfo = get_geoinfo(example_filepath)
    resolution, geo_ex, proj_ex = geoinfo
    print(f"----> resolution is ~{resolution:.0f} meter.")

    for param in spatial_or_constant:
        if isinstance(id[param], type(None)):
            id[param] = np.ones_like(id["ndvi"]) * getattr(c, param)

    if isinstance(id["u_24"], type(None)):
        id["u_24"] = np.sqrt(id["v2m_24"]**2 + id["u2m_24"]**2)

    # Outputs
    od = dict()

    # Constants
    doy = int(date.strftime("%j"))
    sc = ETLook.solar_radiation.seasonal_correction(doy)
    decl = ETLook.solar_radiation.declination(doy)
    iesd = ETLook.solar_radiation.inverse_earth_sun_distance(doy)

    ######################## MODEL ETLOOK ####################################

    # **effective_leaf_area_index*********************************************
    od["vc"] = ETLook.leaf.vegetation_cover(id["ndvi"], c.nd_min, c.nd_max, c.vc_pow)
    od["lai"] = ETLook.leaf.leaf_area_index(od["vc"], c.vc_min, c.vc_max, c.lai_pow)
    od["lai_eff"] = ETLook.leaf.effective_leaf_area_index(od["lai"])
    
    # *******TRANSPIRATION COMPONENT******************************************
    
    # **soil fraction*********************************************************
    od["sf_soil"] = ETLook.radiation.soil_fraction(od["lai"])

    # **atmospheric canopy resistance***********************************************
    od["lat"] = ETLook.solar_radiation.latitude_rad(id["lat_deg"])
    od["ws"] = ETLook.solar_radiation.sunset_hour_angle(od["lat"], decl)
    if isinstance(id["ra_24"], type(None)):
        od["slope"] = ETLook.solar_radiation.slope_rad(id["slope_deg"])
        od["aspect"] = ETLook.solar_radiation.aspect_rad(id["aspect_deg"])
        od["ra_24_toa"] = ETLook.solar_radiation.daily_solar_radiation_toa_new(sc, decl, iesd, od["lat"], doy, od["slope"], od["aspect"])
        od["ra_24_toa_flat"] = ETLook.solar_radiation.daily_solar_radiation_toa_flat(decl, iesd, od["lat"], od["ws"])
        od["diffusion_index"] = ETLook.solar_radiation.diffusion_index(id["trans_24"], c.diffusion_slope, c.diffusion_intercept)
        od["ra_24"] = ETLook.solar_radiation.daily_total_solar_radiation(od["ra_24_toa"], od["ra_24_toa_flat"], od["diffusion_index"], id["trans_24"])
    else:
        od["ra_24"] = np.copy(id["ra_24"])
        # od["ra_24_toa"] = ETLook.solar_radiation.daily_solar_radiation_toa_new(sc, decl, iesd, od["lat"], doy, od["slope"], od["aspect"])
        od["ra_24_toa_flat"] = ETLook.solar_radiation.daily_solar_radiation_toa_flat(decl, iesd, od["lat"], od["ws"])
        od["trans_24"] = od["ra_24"] / od["ra_24_toa_flat"]
        id["trans_24"] = np.copy(od["trans_24"])

    od["stress_rad"] = ETLook.stress.stress_radiation(od["ra_24"])
    id["p_air_0_24"] = ETLook.meteo.air_pressure_kpa2mbar(id["p_air_0_24"])
    od["p_air_24"] = ETLook.meteo.air_pressure_daily(id["z"], id["p_air_0_24"])
    od["vp_24"] = ETLook.meteo.vapour_pressure_from_specific_humidity_daily(id["qv_24"], od["p_air_24"])

    if isinstance(id["t_air_24"], type(None)):
        id["t_air_24"] = np.mean([id["t_air_min_24"], id["t_air_max_24"]], axis = 0)

    if et_look_version == "v2":
        od["svp_24"] = ETLook.meteo.saturated_vapour_pressure(id["t_air_24"])
    elif et_look_version == "dev":
        od["svp_24"] = ETLook.meteo.saturated_vapour_pressure_average(
                    ETLook.meteo.saturated_vapour_pressure_maximum(id["t_air_max_24"]),
                    ETLook.meteo.saturated_vapour_pressure_minimum(id["t_air_min_24"]))

    od["vpd_24"] = ETLook.meteo.vapour_pressure_deficit_daily(od["svp_24"], od["vp_24"])
    od["stress_vpd"] = ETLook.stress.stress_vpd(od["vpd_24"], id["vpd_slope"])
    od["stress_temp"] = ETLook.stress.stress_temperature(id["t_air_24"], id["t_opt"], c.t_min, c.t_max)

    if et_look_version == "dev":
        id["rs_min"] = np.where(id["land_mask"] == 3, 400, 100)

    od["r_canopy_0"] = ETLook.resistance.atmospheric_canopy_resistance(od["lai_eff"], od["stress_rad"], od["stress_vpd"], od["stress_temp"], id["rs_min"], c.rcan_max)

    # **net radiation canopy******************************************************
    od["t_air_k_24"] = ETLook.meteo.air_temperature_kelvin_daily(id["t_air_24"])
    od["l_net"] = ETLook.radiation.longwave_radiation_fao(od["t_air_k_24"], od["vp_24"], id["trans_24"], c.vp_slope, c.vp_offset, id["lw_slope"], id["lw_offset"])
    od["int_mm"] = ETLook.evapotranspiration.interception_mm(id["P_24"], od["vc"], od["lai"], c.int_max)
    od["lh_24"] = ETLook.meteo.latent_heat_daily(id["t_air_24"])
    od["int_wm2"] = ETLook.radiation.interception_wm2(od["int_mm"], od["lh_24"])
    od["rn_24"] = ETLook.radiation.net_radiation(id["r0"], od["ra_24"], od["l_net"], od["int_wm2"])
    od["rn_24_canopy"] = ETLook.radiation.net_radiation_canopy(od["rn_24"], od["sf_soil"])

    # **Soil Moisture Content***********************************************************
    if isinstance(id["se_root"], type(None)):
        if et_look_version == "dev":
            c.z0m_full = 0.04 + 0.01 * (resolution - 30)/(250-30)
            od["lst_zone_mean"] = lst_zone_mean(id, geo_ex, proj_ex, date, folders)
        od, id = se_root(id, od, ETLook, date)
    else:
        od["se_root"] = np.copy(id["se_root"])

    # find water region using ndvi
    if et_look_version == "dev":
        if resolution < 250:
            id["land_mask"] = np.where(id["land_mask"] == 2, 1, id["land_mask"])
            id["land_mask"] = np.where(id["ndvi"] < 0, 2, id["land_mask"])

    od["stress_moist"] = ETLook.stress.stress_moisture(od["se_root"], c.tenacity)
    od["r_canopy"] = ETLook.resistance.canopy_resistance(od["r_canopy_0"], od["stress_moist"], c.rcan_max)

    # **initial canopy aerodynamic resistance***********************************************************
    
    if isinstance(id["z_oro"], type(None)):
        od["slope"] = ETLook.solar_radiation.slope_rad(id["slope_deg"])
        od["z_oro"] = ETLook.roughness.orographic_roughness(od["slope"], resolution)
    else:
        od["z_oro"] = np.copy(id["z_oro"])

    od["z_obst"] = ETLook.roughness.obstacle_height(id["ndvi"], id["z_obst_max"], c.ndvi_obs_min, c.ndvi_obs_max, c.obs_fr)
    od["z0m"] = ETLook.roughness.roughness_length(od["lai"], od["z_oro"], od["z_obst"], id["z_obst_max"], id["land_mask"])
    od["ra_canopy_init"] = ETLook.neutral.initial_canopy_aerodynamic_resistance(id["u_24"], od["z0m"], c.z_obs)

    # **windspeed blending height daily***********************************************************
    od["u_b_24"] = ETLook.meteo.wind_speed_blending_height_daily(id["u_24"], c.z_obs, c.z_b)

    # **ETLook.neutral.initial_daily_transpiration***********************************************************
    od["ad_dry_24"] = ETLook.meteo.dry_air_density_daily(od["p_air_24"], od["vp_24"], od["t_air_k_24"])
    od["ad_moist_24"] = ETLook.meteo.moist_air_density_daily(od["vp_24"], od["t_air_k_24"])
    od["ad_24"] = ETLook.meteo.air_density_daily(od["ad_dry_24"], od["ad_moist_24"])
    od["psy_24"] = ETLook.meteo.psychrometric_constant_daily(od["p_air_24"], od["lh_24"])
    od["ssvp_24"] = ETLook.meteo.slope_saturated_vapour_pressure_daily(id["t_air_24"])
    od["t_24_init"] = ETLook.neutral.initial_daily_transpiration(od["rn_24_canopy"], od["ssvp_24"], od["ad_24"], od["vpd_24"], od["psy_24"], od["r_canopy"], od["ra_canopy_init"])

    # **ETLook.unstable.initial_sensible_heat_flux_canopy_daily***********************************************************
    od["h_canopy_24_init"] = ETLook.unstable.initial_sensible_heat_flux_canopy_daily(od["rn_24_canopy"], od["t_24_init"])

    # **ETLook.unstable.initial_friction_velocity_daily***********************************************************
    od["disp"] = ETLook.roughness.displacement_height(od["lai"], od["z_obst"], id["land_mask"], c.c1)
    od["u_star_24_init"] = ETLook.unstable.initial_friction_velocity_daily(od["u_b_24"], od["z0m"], od["disp"], c.z_b)

    # **ETLook.unstable.transpiration***********************************************************
    od["t_24"] = ETLook.unstable.transpiration(od["rn_24_canopy"], od["ssvp_24"], od["ad_24"], od["vpd_24"], od["psy_24"], od["r_canopy"], od["h_canopy_24_init"], od["t_air_k_24"], od["u_star_24_init"], od["z0m"], od["disp"], od["u_b_24"], c.z_obs, c.z_b, c.iter_h)
    od["t_24_mm"] = ETLook.unstable.transpiration_mm(od["t_24"], od["lh_24"])

    if et_look_version == "dev":
        od["tpot_24"] = ETLook.unstable.transpiration(od["rn_24_canopy"], od["ssvp_24"], od["ad_24"], od["vpd_24"], od["psy_24"], od["r_canopy"] * od["stress_moist"], od["h_canopy_24_init"], od["t_air_k_24"], od["u_star_24_init"], od["z0m"], od["disp"], od["u_b_24"], c.z_obs, c.z_b, c.iter_h)
        od["tpot_24_mm"] = ETLook.unstable.transpiration_mm(od["tpot_24"], od["lh_24"])

    #*******EVAPORATION COMPONENT****************************************************************

    # **ETLook.radiation.net_radiation_soil***********************************************************
    od["rn_24_soil"] = ETLook.radiation.net_radiation_soil(od["rn_24"], od["sf_soil"])

    # **ETLook.resistance.soil_resistance***********************************************************
    od["r_soil"] = ETLook.resistance.soil_resistance(od["se_root"], id["land_mask"], c.r_soil_pow, c.r_soil_min)

    # **ETLook.resistance.soil_resistance***********************************************************
    od["ra_soil_init"] = ETLook.neutral.initial_soil_aerodynamic_resistance(id["u_24"], c.z_obs)

    # **ETLook.unstable.initial_friction_velocity_soil_daily***********************************************************
    od["u_star_24_soil_init"] = ETLook.unstable.initial_friction_velocity_soil_daily(od["u_b_24"], od["disp"], c.z_b)

    # **ETLook.unstable.initial_sensible_heat_flux_soil_daily***********************************************************
    if et_look_version == "dev":
        stc = ETLook.radiation.soil_thermal_conductivity(c.se_top)
        vhc = ETLook.radiation.volumetric_heat_capacity(c.se_top, c.porosity)
    elif et_look_version == "v2":
        stc = ETLook.radiation.soil_thermal_conductivity(od["se_root"])
        vhc = ETLook.radiation.volumetric_heat_capacity(od["se_root"], c.porosity)
       
    dd = ETLook.radiation.damping_depth(stc, vhc)
    od["g0_bs"] = ETLook.radiation.bare_soil_heat_flux(doy, dd, stc, id["t_amp_year"], od["lat"])
    od["g0_24"] = ETLook.radiation.soil_heat_flux(od["g0_bs"], od["sf_soil"], id["land_mask"], od["rn_24_soil"], id["trans_24"], od["ra_24"], od["l_net"], id["rn_slope"], id["rn_offset"])
    od["e_24_init"] = ETLook.neutral.initial_daily_evaporation(od["rn_24_soil"], od["g0_24"], od["ssvp_24"], od["ad_24"], od["vpd_24"], od["psy_24"], od["r_soil"], od["ra_soil_init"])
    od["h_soil_24_init"] = ETLook.unstable.initial_sensible_heat_flux_soil_daily(od["rn_24_soil"], od["e_24_init"], od["g0_24"])

    # **ETLook.unstable.evaporation***********************************************************
    od["e_24"] = ETLook.unstable.evaporation(od["rn_24_soil"], od["g0_24"], od["ssvp_24"], od["ad_24"], od["vpd_24"], od["psy_24"], od["r_soil"], od["h_soil_24_init"], od["t_air_k_24"], od["u_star_24_soil_init"], od["disp"], od["u_b_24"], c.z_b, c.z_obs, c.iter_h)
    od["e_24_mm"] = ETLook.unstable.evaporation_mm(od["e_24"], od["lh_24"])
    od["et_24_mm"] = ETLook.evapotranspiration.et_actual_mm(od["e_24_mm"], od["t_24_mm"])
    
    if et_look_version == "dev":
        od["e_24_mm"][od["e_24_mm"] < 0] = 0
        od["et_24_mm"][od["et_24_mm"] < 0] = 0

    # **ETLook.unstable.evaporation***********************************************************
    od["rn_24_grass"] = ETLook.radiation.net_radiation_grass(od["ra_24"], od["l_net"], c.r0_grass)
    od["et_ref_24"] = ETLook.evapotranspiration.et_reference(od["rn_24_grass"], od["ad_24"], od["psy_24"], od["vpd_24"], od["ssvp_24"], id["u_24"])
    od["et_ref_24_mm"] = ETLook.evapotranspiration.et_reference_mm(od["et_ref_24"], od["lh_24"])

    if et_look_version == "v2":
        od["et_ref_24_mm"][od["et_ref_24_mm"] < 0] = 0

    if et_look_version == "dev":
        eps_a = ETLook.stress.epsilon_autotrophic_respiration()     
        od["lue"] = ETLook.biomass.lue(id["lue_max"], od["stress_temp"], od["stress_moist"], eps_a)
        od["fpar"] = ETLook.leaf.fpar(od["vc"], id["ndvi"])
        od["apar"] = ETLook.leaf.apar(od["ra_24"], od["fpar"])       
        od["biomass_prod"] = ETLook.biomass.biomass(od["apar"], od["lue"])     

    all_files = dict()
    example_filepath = create_fp("r0", vars.inputs["r0"], date, folders)
    geo_ex, proj_ex = get_geoinfo(example_filepath)[1:3]
    for var in vars.outputs.keys():
        if var in od.keys():
            if vars.outputs[var]["output"]:
                PF.Save_as_tiff(vars.outputs[var]["file_path"], od[var], geo_ex, proj_ex)
                all_files[var] = vars.outputs[var]["file_path"]
    return all_files

def se_root(id, od, ETLook, date, verbose = False):

    et_look_version = ETLook.__name__.split(".")[-1].split("_")[-1]

    # Version
    if et_look_version == "v2":
        # ETLook = ETLook_v2
        if not verbose:
            print("--> Running SEroot_v2")
    elif et_look_version == "dev":
        # ETLook = ETLook_dev
        if not verbose:
            print("--> Running SEroot_dev")
    c = ETLook.constants

    if not isinstance(id, dict):
        od = id

    # resolution, geo_ex, proj_ex = geoinfo

    doy = int(date.strftime("%j"))
    sc = ETLook.solar_radiation.seasonal_correction(doy)
    decl = ETLook.solar_radiation.declination(doy)
    day_angle = ETLook.clear_sky_radiation.day_angle(doy)
    dtime = date.hour + (date.minute / 60)

    if "r0_bare" not in id.keys():
        if isinstance(id, xr.Dataset):
            id["r0_bare"] = xr.ones_like(id["ndvi"]) * c.r0_bare
        else:
            id["r0_bare"] = np.ones_like(id["ndvi"]) * c.r0_bare
    if "r0_full" not in id.keys():
        if isinstance(id, xr.Dataset):
            id["r0_full"] = xr.ones_like(id["ndvi"]) * c.r0_full
        else:
            id["r0_full"] = np.ones_like(id["ndvi"]) * c.r0_full

    if et_look_version == "dev":
        id["t_air_i"][id["t_air_i"] < -270] = np.nan
    id["p_air_i"] = ETLook.meteo.air_pressure_kpa2mbar(id["p_air_i"])
    id["p_air_0_i"] = ETLook.meteo.air_pressure_kpa2mbar(id["p_air_0_i"])

    # if et_look_version == "dev":
    #     z0m_full = 0.04 + 0.01 * (resolution - 30)/(250-30)
    # elif et_look_version == "v2":
    #     z0m_full = 0.1

    od["vc"] = ETLook.leaf.vegetation_cover(id["ndvi"], c.nd_min, c.nd_max, c.vc_pow)

    od["t_air_k_i"] = ETLook.meteo.air_temperature_kelvin_inst(id["t_air_i"])
    od["vp_i"] = ETLook.meteo.vapour_pressure_from_specific_humidity_inst(id["qv_i"], id["p_air_i"])
    od["ad_moist_i"] = ETLook.meteo.moist_air_density_inst(od["vp_i"], od["t_air_k_i"])
    od["ad_dry_i"] = ETLook.meteo.dry_air_density_inst(id["p_air_i"], od["vp_i"], od["t_air_k_i"])
    od["ad_i"] = ETLook.meteo.air_density_inst(od["ad_dry_i"], od["ad_moist_i"])
    od["u_b_i_bare"] = ETLook.soil_moisture.wind_speed_blending_height_bare(id["u_i"], c.z0m_bare, c.z_obs, c.z_b)
    od["lon"] = ETLook.solar_radiation.longitude_rad(id["lon_deg"])
    od["lat"] = ETLook.solar_radiation.latitude_rad(id["lat_deg"])
    od["ha"] = ETLook.solar_radiation.hour_angle(sc, dtime, od["lon"])
    
    I0 = ETLook.clear_sky_radiation.solar_constant()
    ied = ETLook.clear_sky_radiation.inverse_earth_sun_distance(day_angle)
    od["h0"] = ETLook.clear_sky_radiation.solar_elevation_angle(od["lat"], decl, od["ha"])
    od["h0ref"] = ETLook.clear_sky_radiation.solar_elevation_angle_refracted(od["h0"])
    od["m"] = ETLook.clear_sky_radiation.relative_optical_airmass(id["p_air_i"], id["p_air_0_i"], od["h0ref"])
    od["rotm"] = ETLook.clear_sky_radiation.rayleigh_optical_thickness(od["m"])
    od["Tl2"] = ETLook.clear_sky_radiation.linke_turbidity(id["wv_i"], c.aod550_i, id["p_air_i"], id["p_air_0_i"])
    G0 = ETLook.clear_sky_radiation.extraterrestrial_irradiance_normal(I0, ied)
    od["B0c"] = ETLook.clear_sky_radiation.beam_irradiance_normal_clear(G0, od["Tl2"], od["m"], od["rotm"], od["h0"])
    od["Bhc"] = ETLook.clear_sky_radiation.beam_irradiance_horizontal_clear(od["B0c"], od["h0"])
    od["Dhc"] = ETLook.clear_sky_radiation.diffuse_irradiance_horizontal_clear(G0, od["Tl2"], od["h0"])
    
    od["ra_hor_clear_i"] = ETLook.clear_sky_radiation.ra_clear_horizontal(od["Bhc"], od["Dhc"])
    od["emiss_atm_i"] = ETLook.soil_moisture.atmospheric_emissivity_inst(od["vp_i"], od["t_air_k_i"])
    
    od["rn_bare"] = ETLook.soil_moisture.net_radiation_bare(od["ra_hor_clear_i"], od["emiss_atm_i"], od["t_air_k_i"], id["lst"], id["r0_bare"])
    od["rn_full"] = ETLook.soil_moisture.net_radiation_full(od["ra_hor_clear_i"], od["emiss_atm_i"], od["t_air_k_i"], id["lst"], id["r0_full"])
    od["h_bare"] = ETLook.soil_moisture.sensible_heat_flux_bare(od["rn_bare"], c.fraction_h_bare)
    od["h_full"] = ETLook.soil_moisture.sensible_heat_flux_full(od["rn_full"], c.fraction_h_full)
    od["u_b_i_full"] = ETLook.soil_moisture.wind_speed_blending_height_full_inst(id["u_i"], c.z0m_full, c.z_obs, c.z_b)
    
    od["u_star_i_bare"] = ETLook.soil_moisture.friction_velocity_bare_inst(od["u_b_i_bare"], c.z0m_bare, c.disp_bare, c.z_b)
    od["u_star_i_full"] = ETLook.soil_moisture.friction_velocity_full_inst(od["u_b_i_full"], c.z0m_full, c.disp_full, c.z_b)
    od["L_bare"] = ETLook.soil_moisture.monin_obukhov_length_bare(od["h_bare"], od["ad_i"], od["u_star_i_bare"], od["t_air_k_i"])
    od["L_full"] = ETLook.soil_moisture.monin_obukhov_length_full(od["h_full"], od["ad_i"], od["u_star_i_full"], od["t_air_k_i"])
    
    od["u_i_soil"] = ETLook.soil_moisture.wind_speed_soil_inst(id["u_i"], od["L_bare"], c.z_obs)
    od["ras"] = ETLook.soil_moisture.aerodynamical_resistance_soil(od["u_i_soil"])
    od["raa"] = ETLook.soil_moisture.aerodynamical_resistance_bare(id["u_i"], od["L_bare"], c.z0m_bare, c.disp_bare, c.z_obs)
    od["rac"] = ETLook.soil_moisture.aerodynamical_resistance_full(id["u_i"], od["L_full"], c.z0m_full, c.disp_full, c.z_obs)
    
    od["t_max_bare"] = ETLook.soil_moisture.maximum_temperature_bare(od["ra_hor_clear_i"], od["emiss_atm_i"], od["t_air_k_i"], od["ad_i"], od["raa"], od["ras"], id["r0_bare"])
    od["t_max_full"] = ETLook.soil_moisture.maximum_temperature_full(od["ra_hor_clear_i"], od["emiss_atm_i"], od["t_air_k_i"], od["ad_i"], od["rac"], id["r0_full"])

    # od["w_i"] = ETLook.soil_moisture.dew_point_temperature_inst(od["vp_i"])
    # od["t_dew_i"] = ETLook.soil_moisture.dew_point_temperature_inst(od["vp_i"])
    # od["t_wet_i"] = ETLook.soil_moisture.wet_bulb_temperature_inst(id["t_air_i"], od["t_dew_i"], id["p_air_i"])
    od["t_wet_i"] = ETLook.soil_moisture.wet_bulb_temperature_inst_new(id["t_air_i"], id["qv_i"], id["p_air_i"])
    #t_wet_i_new2 = ETLook.soil_moisture.wet_bulb_temperature_inst_new2(t_air_i)

    od["lst_max"] = ETLook.soil_moisture.maximum_temperature(od["t_max_bare"], od["t_max_full"], od["vc"])

    if et_look_version == "v2":
        od["t_wet_k_i"] = ETLook.meteo.wet_bulb_temperature_kelvin_inst(od["t_wet_i"])
        od["lst_min"] = ETLook.soil_moisture.minimum_temperature(od["t_wet_k_i"], od["t_air_k_i"], od["vc"])
    elif et_look_version == "dev":
        od["t_min_bare"] = ETLook.soil_moisture.minimum_temperature_bare(od["ra_hor_clear_i"], od["emiss_atm_i"], od["t_air_k_i"], od["ad_i"], od["raa"], od["ras"], od["lst_zone_mean"], c.r0_bare_wet)
        od["t_min_full"] = ETLook.soil_moisture.minimum_temperature_full(od["ra_hor_clear_i"], od["emiss_atm_i"], od["t_air_k_i"], od["ad_i"], od["rac"], od["lst_zone_mean"], id["r0_full"])  
        od["lst_min"] = ETLook.soil_moisture.maximum_temperature(od["t_min_bare"], od["t_min_full"], od["vc"])

    od["se_root"] = ETLook.soil_moisture.soil_moisture_from_maximum_temperature(od["lst_max"], id["lst"], od["lst_min"])

    return od, id

def lst_zone_mean(id, geo_ex, proj_ex, date, folders):
    size_y, size_x = id["ndvi"].shape
    size_y_zone = int(np.ceil(size_y/200))
    size_x_zone = int(np.ceil(size_x/200)) 
    array_fake = np.ones([size_y_zone, size_x_zone])
    geo_new = tuple([geo_ex[0], geo_ex[1] * 200, geo_ex[2], geo_ex[3], geo_ex[4], geo_ex[5]*200])
    MEM_file = PF.Save_as_MEM(array_fake, geo_new, proj_ex)

    LST_filename = create_fp("LST", vars.inputs["LST"], date, folders)
    
    dest_lst_zone_large = PF.reproject_dataset_example(LST_filename, MEM_file, 4)
    lst_zone_mean_large = dest_lst_zone_large.GetRasterBand(1).ReadAsArray()
    lst_zone_mean_large[lst_zone_mean_large==0] = -9999
    lst_zone_mean_large[np.isnan(lst_zone_mean_large)] = -9999

    if np.nanmax(lst_zone_mean_large) == -9999:
        for x in range(0, size_x_zone):
            for y in range(0, size_y_zone):
                lst_zone_mean_large[y, x] = np.nanmean(id["lst"][y*200:np.minimum((y+1)*200, size_y-1), x*200:np.minimum((x+1)*200, size_x-1)])
        lst_zone_mean_large[np.isnan(lst_zone_mean_large)] = -9999

    lst_zone_mean_large = PF.gap_filling(lst_zone_mean_large, -9999, 1)
    dest_lst_zone_large = PF.Save_as_MEM(lst_zone_mean_large, geo_new, proj_ex)
    
    dest_lst_zone = PF.reproject_dataset_example(dest_lst_zone_large, LST_filename, 6)
    lst_zone_mean = dest_lst_zone.GetRasterBand(1).ReadAsArray()
    return lst_zone_mean

# if __name__ == "__main__":

#     project_folder = r"/Volumes/Data/FAO/WaPOR_vs_pyWaPOR/pyWAPOR_v1"
#     startdate = date = "2021-07-01"

#     level = "level_1"
#     et_look_version = "v2"
#     output = None
#     input_data = None

#     # all_files = main(project_folder, startdate)



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: WAPOR/Functions
"""

from pywapor.general import landcover_converter, outputs, variables, tests, waitbar_console, processing_functions, compositer, lapse_rate

__all__ = ['landcover_converter', 'outputs', 'variables', 'tests', 'waitbar_console', 'processing_functions', 'compositer', 'lapse_rate']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Created on Tue Nov  3 11:31:41 2020

@author: timhe
"""


import os
from osgeo import gdal
import datetime
import numpy as np

import watertools.General.data_conversions as DC
import watertools.General.raster_conversions as RC


#LS_folder = r"F:\Project_FAOJORDAN\Input_Data\LS_output_WAPOR"
#composite_folder = r"D:\Project_ISRPAL\SWEO_Input\Variables\ETLook\ETLook_output_composite"
#output_folder = r"F:\Project_FAOJORDAN\Input_Data\LS_output_WAPOR_composite"

#LS_LVL1_maps = os.listdir(composite_folder)
#DATES_LS_LVL1 = np.array([datetime.datetime.strptime(k, "%Y%m%d").toordinal() for k in LS_LVL1_maps if len(k) == 8])

#for DATE_LS_LVL1 in DATES_LS_LVL1:
#    Date = datetime.datetime.fromordinal(DATE_LS_LVL1)
#    if Date.year==2015:
#       main_et(Date, LS_folder, composite_folder, output_folder)
       #main_e(Date, LS_folder, composite_folder, output_folder)
       #main_bio(Date, LS_folder, composite_folder, output_folder)
       #main_sm(Date, LS_folder, composite_folder, output_folder)       
       
def main_et(Date, LS_folder, composite_folder, output_folder):

    date_comp = Date.strftime("%Y%m%d")
    filename_end_out = os.path.join(output_folder, "et_24_mm_%s.tif" %(date_comp))   
 
    if not os.path.exists(filename_end_out):
        
        # Find all LS dates
        LS_LVL3_maps = os.listdir(LS_folder)
        DATES_LS_LVL3 = np.array([datetime.datetime.strptime(k, "%Y%m%d").toordinal() for k in LS_LVL3_maps if len(k) == 8])
        
        # Find amount of days between
        Diff_days = np.abs(DATES_LS_LVL3 - Date.toordinal())
        Diff_days_sorted = np.sort(Diff_days)
        
        # Create LST_rel map
        output_folder_LSTrel = os.path.join(LS_folder, "LSTrel")
        if not os.path.exists(output_folder_LSTrel):
            os.makedirs(output_folder_LSTrel)
            
        # 
        Continue = 1
        
        for Diff_day_sorted in [k for k in Diff_days_sorted if k <60]:
            
            if Continue == 1:
                
                take_days = np.argwhere(Diff_days == Diff_day_sorted)
                
                for take_day in take_days:
                
                    DATE_LS_LVL3 = int(DATES_LS_LVL3[take_day])
                    
                    date_LS = datetime.datetime.fromordinal(DATE_LS_LVL3).strftime("%Y%m%d")
                
                    print("Add %s in relative map" %date_LS)
                    
                    Output_filename_LSTrel = os.path.join(output_folder_LSTrel, "lstrel_%s.tif" %date_LS)
                    
                    if not os.path.exists(Output_filename_LSTrel):
                    
                        ET_map_LS = os.path.join(LS_folder, date_LS, "et_24_mm_%s.tif" %(date_LS))
                        
                        dest_LS_30 = gdal.Open(ET_map_LS)
                        geo_30 = dest_LS_30.GetGeoTransform()
                        proj = dest_LS_30.GetProjection()
                        size_x = dest_LS_30.RasterXSize
                        size_y = dest_LS_30.RasterYSize
                        Array_LS = dest_LS_30.GetRasterBand(1).ReadAsArray()
                        
                        geo_1000 = np.array(geo_30)
                        geo_1000[1] = geo_1000[1]*108
                        geo_1000[5] = geo_1000[5]*108
                        geo_1000 = tuple(geo_1000)
                        
                        # Create MEM up file
                        dummy_array = np.ones([int(np.floor(size_y/108)),int(np.floor(size_x/108))]) * np.nan
                        dest_up = DC.Save_as_MEM(dummy_array, geo_1000, 4326)
                        
                        # upscale LS
                        # dest_LS_up = RC.reproject_dataset_example(dest_LS_30, dest_up, 4)
            
                        # Get relative array
                        relMap_LS_one = Create_Relative_Map(Array_LS, 108, Include_NAN = 0)  
                        
                        DC.Save_as_tiff(Output_filename_LSTrel, relMap_LS_one, geo_30, proj)
                    
                    else:
                        dest_lstrel = gdal.Open(Output_filename_LSTrel)
                        relMap_LS_one = dest_lstrel.GetRasterBand(1).ReadAsArray()
                    
                    if "relMap_LS" not in locals():
                        relMap_LS = np.ones(relMap_LS_one.shape) * np.nan
                        
                    # Fit in endmap
                    relMap_LS = np.where(np.isnan(relMap_LS), relMap_LS_one, relMap_LS)
                
                    Amount_Of_Pixels2Go = np.count_nonzero(np.isnan(relMap_LS))
                
                if Amount_Of_Pixels2Go == 0:
                    Continue = 0
                else:
                    print("Still %s pixels to go" %Amount_Of_Pixels2Go)

        if "dest_up" not in locals():
            
            DATE_LS_LVL3 = int(DATES_LS_LVL3[0])   
            date_LS = datetime.datetime.fromordinal(DATE_LS_LVL3).strftime("%Y%m%d")           
            ET_map_LS = os.path.join(LS_folder, date_LS, "et_24_mm_%s.tif" %(date_LS))
                        
            dest_LS_30 = gdal.Open(ET_map_LS)
            geo_30 = dest_LS_30.GetGeoTransform()
            proj = dest_LS_30.GetProjection()
            size_x = dest_LS_30.RasterXSize
            size_y = dest_LS_30.RasterYSize
            Array_LS = dest_LS_30.GetRasterBand(1).ReadAsArray()
            
            geo_1000 = np.array(geo_30)
            geo_1000[1] = geo_1000[1]*108
            geo_1000[5] = geo_1000[5]*108
            geo_1000 = tuple(geo_1000)
            
            # Create MEM up file
            dummy_array = np.ones([int(np.floor(size_y/108)),int(np.floor(size_x/108))]) * np.nan
            dest_up = DC.Save_as_MEM(dummy_array, geo_1000, 4326)   
         
        ET_map_Comp = os.path.join(composite_folder, date_comp, "et_24_mm_%s.tif" %(date_comp)) 
        dest_comp_up = RC.reproject_dataset_example(ET_map_Comp, dest_up, 4) 
        
        # Downscale back to 30m
        dest_comp_down = RC.reproject_dataset_example(dest_comp_up, dest_LS_30, 2) 
        Array_comp = dest_comp_down.GetRasterBand(1).ReadAsArray()
                                  
        Temp_array = relMap_LS.clip(0.0001, 6) * Array_comp
        
        # Scale Temp_array
        dest_mem_temp = DC.Save_as_MEM(Temp_array, geo_30, 4326)
        
        # Open input composite and get geotransform
        dest_comp = gdal.Open(ET_map_Comp)
        geo_comp = dest_comp.GetGeoTransform()
        proj_comp = dest_comp.GetProjection()
        Array_comp_MODIS = dest_comp.GetRasterBand(1).ReadAsArray()
        Array_comp_MODIS_BOX = Create_Relative_Map(Array_comp_MODIS, 3, Include_NAN = 1, type_return = "mean")
        
        # Upscale LS comp
        dest_LScomp_up = RC.reproject_dataset_example(dest_mem_temp, ET_map_Comp, 4)
        Array_LScomp_up = dest_LScomp_up.GetRasterBand(1).ReadAsArray()
        Array_LScomp_up_BOX = Create_Relative_Map(Array_LScomp_up, 3, Include_NAN = 1, type_return = "mean")
        
        # Calculate Fraction Correction
        Corr_Frac = Array_comp_MODIS_BOX/Array_LScomp_up_BOX
        Corr_Frac = Corr_Frac.clip(0.0001, 6)
        dest_corr_up = DC.Save_as_MEM(Corr_Frac, geo_comp, proj_comp)
        
        # downscale corr factor
        dest_corr_down = RC.reproject_dataset_example(dest_corr_up, dest_mem_temp, 2)
        Corr_down = dest_corr_down.GetRasterBand(1).ReadAsArray()
        
        # apply corr
        End_array = Corr_down * Temp_array
        End_array[End_array>11] = np.nan
        
        DC.Save_as_tiff(filename_end_out, End_array, geo_30, 4326)
    
    return()
                
  
def main_e(Date, LS_folder, composite_folder, output_folder):

    date_comp = Date.strftime("%Y%m%d")
    filename_end_out = os.path.join(output_folder, "e_24_mm_%s.tif" %(date_comp))   
 
    if not os.path.exists(filename_end_out):
        
        # Find all LS dates
        LS_LVL3_maps = os.listdir(LS_folder)
        DATES_LS_LVL3 = np.array([datetime.datetime.strptime(k, "%Y%m%d").toordinal() for k in LS_LVL3_maps if len(k) == 8])
        
        # Find amount of days between
        Diff_days = np.abs(DATES_LS_LVL3 - Date.toordinal())
        Diff_days_sorted = np.sort(Diff_days)
        
        # Create E_rel map
        output_folder_Erel = os.path.join(LS_folder, "Erel")
        if not os.path.exists(output_folder_Erel):
            os.makedirs(output_folder_Erel)
            
        # 
        Continue = 1
        
        for Diff_day_sorted in [k for k in Diff_days_sorted if k <60]:
            
            if Continue == 1:
                
                take_days = np.argwhere(Diff_days == Diff_day_sorted)
                
                for take_day in take_days:
                
                    DATE_LS_LVL3 = int(DATES_LS_LVL3[take_day])
                    
                    date_LS = datetime.datetime.fromordinal(DATE_LS_LVL3).strftime("%Y%m%d")
                
                    print("Add %s in relative map" %date_LS)
                    
                    Output_filename_Erel = os.path.join(output_folder_Erel, "erel_%s.tif" %date_LS)
                    
                    if not os.path.exists(Output_filename_Erel):
                    
                        E_map_LS = os.path.join(LS_folder, date_LS, "e_24_mm_%s.tif" %(date_LS))
                        
                        dest_LS_30 = gdal.Open(E_map_LS)
                        geo_30 = dest_LS_30.GetGeoTransform()
                        proj = dest_LS_30.GetProjection()
                        size_x = dest_LS_30.RasterXSize
                        size_y = dest_LS_30.RasterYSize
                        Array_LS = dest_LS_30.GetRasterBand(1).ReadAsArray()
                        
                        geo_1000 = np.array(geo_30)
                        geo_1000[1] = geo_1000[1]*108
                        geo_1000[5] = geo_1000[5]*108
                        geo_1000 = tuple(geo_1000)
                        
                        # Create MEM up file
                        dummy_array = np.ones([int(np.floor(size_y/108)),int(np.floor(size_x/108))]) * np.nan
                        dest_up = DC.Save_as_MEM(dummy_array, geo_1000, 4326)
                        
                        # upscale LS
                        dest_LS_up = RC.reproject_dataset_example(dest_LS_30, dest_up, 4)
            
                        # Get relative array
                        relMap_LS_one = Create_Relative_Map(Array_LS, 108, Include_NAN = 0)  
                        
                        DC.Save_as_tiff(Output_filename_Erel, relMap_LS_one, geo_30, proj)
                    
                    else:
                        dest_erel = gdal.Open(Output_filename_Erel)
                        relMap_LS_one = dest_erel.GetRasterBand(1).ReadAsArray()
                    
                    if "relMap_LS" not in locals():
                        relMap_LS = np.ones(relMap_LS_one.shape) * np.nan
                        
                    # Fit in endmap
                    relMap_LS = np.where(np.isnan(relMap_LS), relMap_LS_one, relMap_LS)
                
                    Amount_Of_Pixels2Go = np.count_nonzero(np.isnan(relMap_LS))
                
                if Amount_Of_Pixels2Go == 0:
                    Continue = 0
                else:
                    print("Still %s pixels to go" %Amount_Of_Pixels2Go)
         
        if "dest_up" not in locals():
            
            DATE_LS_LVL3 = int(DATES_LS_LVL3[0])   
            date_LS = datetime.datetime.fromordinal(DATE_LS_LVL3).strftime("%Y%m%d")           
            E_map_LS = os.path.join(LS_folder, date_LS, "e_24_mm_%s.tif" %(date_LS))
                        
            dest_LS_30 = gdal.Open(E_map_LS)
            geo_30 = dest_LS_30.GetGeoTransform()
            proj = dest_LS_30.GetProjection()
            size_x = dest_LS_30.RasterXSize
            size_y = dest_LS_30.RasterYSize
            Array_LS = dest_LS_30.GetRasterBand(1).ReadAsArray()
            
            geo_1000 = np.array(geo_30)
            geo_1000[1] = geo_1000[1]*108
            geo_1000[5] = geo_1000[5]*108
            geo_1000 = tuple(geo_1000)
            
            # Create MEM up file
            dummy_array = np.ones([int(np.floor(size_y/108)),int(np.floor(size_x/108))]) * np.nan
            dest_up = DC.Save_as_MEM(dummy_array, geo_1000, 4326)   
         
        E_map_Comp = os.path.join(composite_folder, date_comp, "e_24_mm_%s.tif" %(date_comp)) 
        dest_comp_up = RC.reproject_dataset_example(E_map_Comp, dest_up, 4) 
        
        # Downscale back to 30m
        dest_comp_down = RC.reproject_dataset_example(dest_comp_up, dest_LS_30, 2) 
        Array_comp = dest_comp_down.GetRasterBand(1).ReadAsArray()
                                  
        Temp_array = relMap_LS.clip(0.0001, 6) * Array_comp
        
        # Scale Temp_array
        dest_mem_temp = DC.Save_as_MEM(Temp_array, geo_30, 4326)
        
        # Open input composite and get geotransform
        dest_comp = gdal.Open(E_map_Comp)
        geo_comp = dest_comp.GetGeoTransform()
        proj_comp = dest_comp.GetProjection()
        Array_comp_MODIS = dest_comp.GetRasterBand(1).ReadAsArray()
        Array_comp_MODIS_BOX = Create_Relative_Map(Array_comp_MODIS, 3, Include_NAN = 1, type_return = "mean")
        
        # Upscale LS comp
        dest_LScomp_up = RC.reproject_dataset_example(dest_mem_temp, E_map_Comp, 4)
        Array_LScomp_up = dest_LScomp_up.GetRasterBand(1).ReadAsArray()
        Array_LScomp_up_BOX = Create_Relative_Map(Array_LScomp_up, 3, Include_NAN = 1, type_return = "mean")
        
        # Calculate Fraction Correction
        Corr_Frac = Array_comp_MODIS_BOX/Array_LScomp_up_BOX
        Corr_Frac = Corr_Frac.clip(0.0001, 6)
        dest_corr_up = DC.Save_as_MEM(Corr_Frac, geo_comp, proj_comp)
        
        # downscale corr factor
        dest_corr_down = RC.reproject_dataset_example(dest_corr_up, dest_mem_temp, 2)
        Corr_down = dest_corr_down.GetRasterBand(1).ReadAsArray()
        
        # apply corr
        End_array = Corr_down * Temp_array
        End_array[End_array>11] = np.nan
        
        DC.Save_as_tiff(filename_end_out, End_array, geo_30, 4326)
    
    return()
                

            
def main_bio(Date, LS_folder, composite_folder, output_folder):

    date_comp = Date.strftime("%Y%m%d")
    filename_end_out = os.path.join(output_folder, "biomass_prod_kg-ha_%s.tif" %(date_comp))   
 
    if not os.path.exists(filename_end_out):
        
        # Find all LS dates
        LS_LVL3_maps = os.listdir(LS_folder)
        DATES_LS_LVL3 = np.array([datetime.datetime.strptime(k, "%Y%m%d").toordinal() for k in LS_LVL3_maps if len(k) == 8])
        
        # Find amount of days between
        Diff_days = np.abs(DATES_LS_LVL3 - Date.toordinal())
        Diff_days_sorted = np.sort(Diff_days)
        
        # Create BIO_rel map
        output_folder_BIOrel = os.path.join(LS_folder, "BIOrel")
        if not os.path.exists(output_folder_BIOrel):
            os.makedirs(output_folder_BIOrel)
            
        # 
        Continue = 1
        
        for Diff_day_sorted in [k for k in Diff_days_sorted if k <60]:
            
            if Continue == 1:
                
                take_days = np.argwhere(Diff_days == Diff_day_sorted)
                
                for take_day in take_days:
                
                    DATE_LS_LVL3 = int(DATES_LS_LVL3[take_day])
                    
                    date_LS = datetime.datetime.fromordinal(DATE_LS_LVL3).strftime("%Y%m%d")
                
                    print("Add %s in relative map" %date_LS)
                    
                    Output_filename_BIOrel = os.path.join(output_folder_BIOrel, "biorel_%s.tif" %date_LS)
                    
                    if not os.path.exists(Output_filename_BIOrel):
                    
                        BIO_map_LS = os.path.join(LS_folder, date_LS, "biomass_prod_kg-ha_%s.tif" %(date_LS))
                        
                        dest_LS_30 = gdal.Open(BIO_map_LS)
                        geo_30 = dest_LS_30.GetGeoTransform()
                        proj = dest_LS_30.GetProjection()
                        size_x = dest_LS_30.RasterXSize
                        size_y = dest_LS_30.RasterYSize
                        Array_LS = dest_LS_30.GetRasterBand(1).ReadAsArray()
                        
                        geo_1000 = np.array(geo_30)
                        geo_1000[1] = geo_1000[1]*108
                        geo_1000[5] = geo_1000[5]*108
                        geo_1000 = tuple(geo_1000)
                        
                        # Create MEM up file
                        dummy_array = np.ones([int(np.floor(size_y/108)),int(np.floor(size_x/108))]) * np.nan
                        dest_up = DC.Save_as_MEM(dummy_array, geo_1000, 4326)
                        
                        # upscale LS
                        dest_LS_up = RC.reproject_dataset_example(dest_LS_30, dest_up, 4)
            
                        # Get relative array
                        relMap_LS_one = Create_Relative_Map(Array_LS, 108, Include_NAN = 0)  
                        
                        DC.Save_as_tiff(Output_filename_BIOrel, relMap_LS_one, geo_30, proj)
                    
                    else:
                        dest_biorel = gdal.Open(Output_filename_BIOrel)
                        relMap_LS_one = dest_biorel.GetRasterBand(1).ReadAsArray()
                    
                    if "relMap_LS" not in locals():
                        relMap_LS = np.ones(relMap_LS_one.shape) * np.nan
                        
                    # Fit in endmap
                    relMap_LS = np.where(np.isnan(relMap_LS), relMap_LS_one, relMap_LS)
                
                    Amount_Of_Pixels2Go = np.count_nonzero(np.isnan(relMap_LS))
                
                if Amount_Of_Pixels2Go == 0:
                    Continue = 0
                else:
                    print("Still %s pixels to go" %Amount_Of_Pixels2Go)
         
        if "dest_up" not in locals():
            
            DATE_LS_LVL3 = int(DATES_LS_LVL3[0])   
            date_LS = datetime.datetime.fromordinal(DATE_LS_LVL3).strftime("%Y%m%d")           
            BIO_map_LS = os.path.join(LS_folder, date_LS, "biomass_prod_kg-ha_%s.tif" %(date_LS))
                        
            dest_LS_30 = gdal.Open(BIO_map_LS)
            geo_30 = dest_LS_30.GetGeoTransform()
            proj = dest_LS_30.GetProjection()
            size_x = dest_LS_30.RasterXSize
            size_y = dest_LS_30.RasterYSize
            Array_LS = dest_LS_30.GetRasterBand(1).ReadAsArray()
            
            geo_1000 = np.array(geo_30)
            geo_1000[1] = geo_1000[1]*108
            geo_1000[5] = geo_1000[5]*108
            geo_1000 = tuple(geo_1000)
            
            # Create MEM up file
            dummy_array = np.ones([int(np.floor(size_y/108)),int(np.floor(size_x/108))]) * np.nan
            dest_up = DC.Save_as_MEM(dummy_array, geo_1000, 4326)   
         
        BIO_map_Comp = os.path.join(composite_folder, date_comp, "biomass_prod_kg-ha_%s.tif" %(date_comp)) 
        dest_comp_up = RC.reproject_dataset_example(BIO_map_Comp, dest_up, 4) 
        
        # Downscale back to 30m
        dest_comp_down = RC.reproject_dataset_example(dest_comp_up, dest_LS_30, 2) 
        Array_comp = dest_comp_down.GetRasterBand(1).ReadAsArray()
                                  
        Temp_array = relMap_LS.clip(0.0001, 6) * Array_comp
        
        # Scale Temp_array
        dest_mem_temp = DC.Save_as_MEM(Temp_array, geo_30, 4326)
        
        # Open input composite and get geotransform
        dest_comp = gdal.Open(BIO_map_Comp)
        geo_comp = dest_comp.GetGeoTransform()
        proj_comp = dest_comp.GetProjection()
        Array_comp_MODIS = dest_comp.GetRasterBand(1).ReadAsArray()
        Array_comp_MODIS_BOX = Create_Relative_Map(Array_comp_MODIS, 3, Include_NAN = 1, type_return = "mean")
        
        # Upscale LS comp
        dest_LScomp_up = RC.reproject_dataset_example(dest_mem_temp, BIO_map_Comp, 4)
        Array_LScomp_up = dest_LScomp_up.GetRasterBand(1).ReadAsArray()
        Array_LScomp_up_BOX = Create_Relative_Map(Array_LScomp_up, 3, Include_NAN = 1, type_return = "mean")
        
        # Calculate Fraction Correction
        Corr_Frac = Array_comp_MODIS_BOX/Array_LScomp_up_BOX
        Corr_Frac = Corr_Frac.clip(0.0001, 6)
        dest_corr_up = DC.Save_as_MEM(Corr_Frac, geo_comp, proj_comp)
        
        # downscale corr factor
        dest_corr_down = RC.reproject_dataset_example(dest_corr_up, dest_mem_temp, 2)
        Corr_down = dest_corr_down.GetRasterBand(1).ReadAsArray()
        
        # apply corr
        End_array = Corr_down * Temp_array
        End_array[End_array>300] = 300
        
        DC.Save_as_tiff(filename_end_out, End_array, geo_30, 4326)
    
    return()
                            
            
        
def main_sm(Date, LS_folder, composite_folder, output_folder):

    date_comp = Date.strftime("%Y%m%d")
    filename_end_out = os.path.join(output_folder, "se_root_%s.tif" %(date_comp))   
 
    if not os.path.exists(filename_end_out):
        
        # Find all LS dates
        LS_LVL3_maps = os.listdir(LS_folder)
        DATES_LS_LVL3 = np.array([datetime.datetime.strptime(k, "%Y%m%d").toordinal() for k in LS_LVL3_maps if len(k) == 8])
        
        # Find amount of days between
        Diff_days = np.abs(DATES_LS_LVL3 - Date.toordinal())
        Diff_days_sorted = np.sort(Diff_days)
        
        # Create SE_rel map
        output_folder_SErel = os.path.join(LS_folder, "SErel")
        if not os.path.exists(output_folder_SErel):
            os.makedirs(output_folder_SErel)
            
        # 
        Continue = 1
        
        for Diff_day_sorted in [k for k in Diff_days_sorted if k <60]:
            
            if Continue == 1:
                
                take_days = np.argwhere(Diff_days == Diff_day_sorted)
                
                for take_day in take_days:
                
                    DATE_LS_LVL3 = int(DATES_LS_LVL3[take_day])
                    
                    date_LS = datetime.datetime.fromordinal(DATE_LS_LVL3).strftime("%Y%m%d")
                
                    print("Add %s in relative map" %date_LS)
                    
                    Output_filename_SErel = os.path.join(output_folder_SErel, "serel_%s.tif" %date_LS)
                    
                    if not os.path.exists(Output_filename_SErel):
                    
                        SE_map_LS = os.path.join(LS_folder, date_LS, "se_root_%s.tif" %(date_LS))
                        
                        dest_LS_30 = gdal.Open(SE_map_LS)
                        geo_30 = dest_LS_30.GetGeoTransform()
                        proj = dest_LS_30.GetProjection()
                        size_x = dest_LS_30.RasterXSize
                        size_y = dest_LS_30.RasterYSize
                        Array_LS = dest_LS_30.GetRasterBand(1).ReadAsArray()
                        
                        geo_1000 = np.array(geo_30)
                        geo_1000[1] = geo_1000[1]*108
                        geo_1000[5] = geo_1000[5]*108
                        geo_1000 = tuple(geo_1000)
                        
                        # Create MEM up file
                        dummy_array = np.ones([int(np.floor(size_y/108)),int(np.floor(size_x/108))]) * np.nan
                        dest_up = DC.Save_as_MEM(dummy_array, geo_1000, 4326)
                        
                        # upscale LS
                        dest_LS_up = RC.reproject_dataset_example(dest_LS_30, dest_up, 4)
            
                        # Get relative array
                        relMap_LS_one = Create_Relative_Map(Array_LS, 108, Include_NAN = 0)  
                        
                        DC.Save_as_tiff(Output_filename_SErel, relMap_LS_one, geo_30, proj)
                    
                    else:
                        dest_serel = gdal.Open(Output_filename_SErel)
                        relMap_LS_one = dest_serel.GetRasterBand(1).ReadAsArray()
                    
                    if "relMap_LS" not in locals():
                        relMap_LS = np.ones(relMap_LS_one.shape) * np.nan
                        
                    # Fit in endmap
                    relMap_LS = np.where(np.isnan(relMap_LS), relMap_LS_one, relMap_LS)
                
                    Amount_Of_Pixels2Go = np.count_nonzero(np.isnan(relMap_LS))
                
                if Amount_Of_Pixels2Go == 0:
                    Continue = 0
                else:
                    print("Still %s pixels to go" %Amount_Of_Pixels2Go)
         
        if "dest_up" not in locals():
            
            DATE_LS_LVL3 = int(DATES_LS_LVL3[0])   
            date_LS = datetime.datetime.fromordinal(DATE_LS_LVL3).strftime("%Y%m%d")           
            SE_map_LS = os.path.join(LS_folder, date_LS, "se_root_%s.tif" %(date_LS))
                        
            dest_LS_30 = gdal.Open(SE_map_LS)
            geo_30 = dest_LS_30.GetGeoTransform()
            proj = dest_LS_30.GetProjection()
            size_x = dest_LS_30.RasterXSize
            size_y = dest_LS_30.RasterYSize
            Array_LS = dest_LS_30.GetRasterBand(1).ReadAsArray()
            
            geo_1000 = np.array(geo_30)
            geo_1000[1] = geo_1000[1]*108
            geo_1000[5] = geo_1000[5]*108
            geo_1000 = tuple(geo_1000)
            
            # Create MEM up file
            dummy_array = np.ones([int(np.floor(size_y/108)),int(np.floor(size_x/108))]) * np.nan
            dest_up = DC.Save_as_MEM(dummy_array, geo_1000, 4326)   
         
        SE_map_Comp = os.path.join(composite_folder, date_comp, "se_root_%s.tif" %(date_comp)) 
        dest_comp_up = RC.reproject_dataset_example(SE_map_Comp, dest_up, 4) 
        
        # Downscale back to 30m
        dest_comp_down = RC.reproject_dataset_example(dest_comp_up, dest_LS_30, 2) 
        Array_comp = dest_comp_down.GetRasterBand(1).ReadAsArray()
                                  
        Temp_array = relMap_LS.clip(0.0001, 6) * Array_comp
        
        # Scale Temp_array
        dest_mem_temp = DC.Save_as_MEM(Temp_array, geo_30, 4326)
        
        # Open input composite and get geotransform
        dest_comp = gdal.Open(SE_map_Comp)
        geo_comp = dest_comp.GetGeoTransform()
        proj_comp = dest_comp.GetProjection()
        Array_comp_MODIS = dest_comp.GetRasterBand(1).ReadAsArray()
        Array_comp_MODIS_BOX = Create_Relative_Map(Array_comp_MODIS, 3, Include_NAN = 1, type_return = "mean")
        
        # Upscale LS comp
        dest_LScomp_up = RC.reproject_dataset_example(dest_mem_temp, SE_map_Comp, 4)
        Array_LScomp_up = dest_LScomp_up.GetRasterBand(1).ReadAsArray()
        Array_LScomp_up_BOX = Create_Relative_Map(Array_LScomp_up, 3, Include_NAN = 1, type_return = "mean")
        
        # Calculate Fraction Correction
        Corr_Frac = Array_comp_MODIS_BOX/Array_LScomp_up_BOX
        Corr_Frac = Corr_Frac.clip(0.0001, 6)
        dest_corr_up = DC.Save_as_MEM(Corr_Frac, geo_comp, proj_comp)
        
        # downscale corr factor
        dest_corr_down = RC.reproject_dataset_example(dest_corr_up, dest_mem_temp, 2)
        Corr_down = dest_corr_down.GetRasterBand(1).ReadAsArray()
        
        # apply corr
        End_array = Corr_down * Temp_array
        End_array=End_array.clip(0,1)
        
        DC.Save_as_tiff(filename_end_out, End_array, geo_30, 4326)
    
    return()
                            
            
            
                   
        
    
   





'''
dest_out = RC.reproject_dataset_example(r"D:\Project_FAO\Test_LVL3\ET_LSCOMP_30m.tif", ET_map_Comp, 4)
array_out = dest_out.GetRasterBand(1).ReadAsArray()
dest_ex = gdal.Open(ET_map_Comp)
geo_ex = dest_ex.GetGeoTransform()
DC.Save_as_tiff(r"D:\Project_FAO\Test_LVL3\ET_LSCOMP_250m.tif", array_out, geo_ex, 4326)

Array_Comp = dest_comp_up.GetRasterBand(1).ReadAsArray()
Array_LS = dest_LS_up.GetRasterBand(1).ReadAsArray()

dest_LSCOMP_up = RC.reproject_dataset_example(r"D:\Project_FAO\Test_LVL3\ET_LSCOMP_30m.tif", dest_comp_up, 4)
Array_LSCOMP = dest_LSCOMP_up.GetRasterBand(1).ReadAsArray()
DC.Save_as_tiff(r"D:\Project_FAO\Test_LVL3\ET_Comp_1000m.tif", Array_Comp, geo_1000, 4326)
DC.Save_as_tiff(r"D:\Project_FAO\Test_LVL3\ET_LS_1000m.tif", Array_LS, geo_1000, 4326)
DC.Save_as_tiff(r"D:\Project_FAO\Test_LVL3\ET_LSCOMP_1000m.tif", Array_LSCOMP, geo_1000, 4326)


print("ET Average COMPOSITE = %s" %np.nanmean(Array_Comp))
print("ET Average LS = %s"%np.nanmean(Array_LS))
print("ET Average LS_COMPOSITE = %s"%np.nanmean(Array_LSCOMP))

'''










def Create_Relative_Map(Data_In, Box, Include_NAN = 1, type_return = "relative"):
    
    # Define required parameters
    Data_In[Data_In==-9999] = np.nan
    Size_box = int((Box-1)/2)
    Data_Out=np.zeros([len(Data_In),len(Data_In[1])]) 
    Data_amount = np.zeros([len(Data_In),len(Data_In[1])]) 

    # create 3D box of the moving window
    for ypixel in list(range(-Size_box,Size_box + 1)):
        for xpixel in list(range(-Size_box,Size_box + 1)):
           
            Xmin = np.maximum(0, xpixel)
            Xmax = np.minimum(len(Data_In[1]) + xpixel, len(Data_In[1]))
            Ymin = np.maximum(0, ypixel)
            Ymax = np.minimum(len(Data_In) + ypixel, len(Data_In))
            
            Xmin2 = len(Data_In[1]) - Xmax
            Xmax2 = len(Data_In[1]) - Xmin
            Ymin2 = len(Data_In) - Ymax
            Ymax2 = len(Data_In) - Ymin
            
            if Include_NAN == 1:
                Data_in_one = Data_In[Ymin:Ymax, Xmin:Xmax]
                Data_Out[Ymin2:Ymax2, Xmin2:Xmax2] += np.where(np.isnan(Data_in_one), 0, Data_in_one)
                
                Data_amount_one = np.where(np.isnan(Data_in_one), 0, 1)
                Data_amount[Ymin2:Ymax2, Xmin2:Xmax2]  += Data_amount_one
            else:
                Data_Out[Ymin2:Ymax2, Xmin2:Xmax2] += Data_In[Ymin:Ymax, Xmin:Xmax]
                Data_amount[Ymin2:Ymax2, Xmin2:Xmax2] += 1 
                     
    Data_mean = Data_Out/Data_amount

    if type_return == "relative":
        # Calculate the relative map
        End = Data_In/Data_mean
    else:
        End = Data_mean

    return(End)



from setuptools import setup, find_packages

setup(
    name = 'pywapor',
    version = '2.3.4',
    url = 'https://bitbucket.org/cioapps/wapor-et-look/src/master/',
    author = "FAO",
    author_email = "bert.coerver@fao.org",
    license = "Apache",
    packages = find_packages(include = ['pywapor', 'pywapor.*']),
    include_package_data=True,
    install_requires = [
        'gdal<=3.1.4',
        'aiohttp==3.7.4.post0',
        'numpy',
        'pandas',
        'requests',
        'matplotlib',
        'netcdf4',
        'pyproj',
        'scipy',
        'fiona',
        'pycurl',
        'pyshp',
        'joblib',
        'bs4',
        'paramiko',
        'rasterio',
        'xarray>=0.20.0',
        'geojson',
        'vito_download',
        'nest_asyncio',
        'tqdm',
        'dask',
        'rioxarray',
    ]
)


import sys
from pywapor.collect.MCD43.DataAccess import DownloadData
import os
import glob
import pywapor

def main(Dir, latlim, lonlim, Startdate, Enddate, Waitbar = 1, hdf_library = None, remove_hdf = 1):
    """
    This function downloads MCD43 albedo daily data for the specified time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    username -- "" string giving the username of your NASA account (https://urs.earthdata.nasa.gov/)
    password -- "" string giving the password of your NASA account    
    Waitbar -- 1 (Default) will print a waitbar
    hdf_library -- string, if all the hdf files are already stored on computer
                    define directory to the data here
    remove_hdf -- 1 (Default), if 1 remove all the downloaded hdf files in the end    
    """
    username, password = pywapor.collect.get_pw_un.get("NASA")

    print('\nDownload daily MODIS Albedo data for period %s till %s' %(Startdate, Enddate))
    DownloadData(Dir, Startdate, Enddate, latlim, lonlim, username, password, Waitbar, hdf_library, remove_hdf)

    output_files = glob.glob(os.path.join(Dir, 'MODIS', 'MCD43', "*.tif"))

    return output_files

if __name__ == '__main__':
    main(sys.argv)


# -*- coding: utf-8 -*-
"""
WaterSat
author: Tim Martijn Hessels
Created on Tue Feb 19 08:47:49 2019
"""

import os
import numpy as np
import pandas as pd
import urllib

def DownloadData(Dir, Var, Startdate, Enddate, latlim, lonlim, TimeStep, Period, Waitbar):

    if TimeStep == "daily" and Var == "t2m":
        all_files = {Var: list(), "t2m-min": list(), "t2m-max": list()}
    else:
        all_files = {Var: list()}

    # Add extra buffer to ensure good spatial interpolation
    buffer_pixels = 0
    lonlim = [lonlim[0] - 0.3125 * buffer_pixels, lonlim[1] + 0.3125 * buffer_pixels]
    latlim = [latlim[0] - 0.25 * buffer_pixels, latlim[1] + 0.25 * buffer_pixels]

	# WAPOR modules
    import pywapor.general.processing_functions as PF

    # Check the latitude and longitude and otherwise set lat or lon on greatest extent
    if latlim[0] < -90 or latlim[1] > 90:
        print('Latitude above 90N or below 90S is not possible. Value set to maximum')
        latlim[0] = np.max(latlim[0], -90)
        latlim[1] = np.min(latlim[1], 90)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W. Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0], -180)
        lonlim[1] = np.min(lonlim[1], 180)

    # Get information of the parameter
    VarInfo = VariablesInfo(TimeStep)
    Parameter = VarInfo.names[Var]
    unit  = VarInfo.units[Var]
    types  = VarInfo.types[Var]

    # Create output folder
    output_folder = os.path.join(Dir, "GEOS5", Parameter, TimeStep)
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Define IDs
    IDx = [np.floor((lonlim[0] + 180)/0.3125), np.ceil((lonlim[1] + 180)/0.3125)]
    IDy = [np.floor((latlim[0] + 90)/0.25), np.ceil((latlim[1] + 90)/0.25)]

    # Create output geo transform
    Xstart = -180 + 0.3125 * IDx[0]
    Ystart = -90 + 0.25 * IDy[1]
    geo_out = tuple([Xstart, 0.3125, 0, Ystart, 0, -0.25])
    proj = "WGS84"

    Dates = pd.date_range(Startdate, Enddate, freq = "D")

    # Create Waitbar
    # if Waitbar == 1:
    #     import pywapor.general.waitbar_console as WaitbarConsole
    #     total_amount = len(Dates)
    #     amount = 0
    #     WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    for Date in Dates:

        # Define the IDz
        if TimeStep == "three_hourly":
            IDz_start = IDz_end = int(((Date - pd.Timestamp("2017-12-01")).days) * 8) + (Period - 1)
            Hour = int((Period - 1) * 3)
            output_name = os.path.join(output_folder, "%s_GEOS_%s_3-hourly_%d.%02d.%02d_H%02d.M00.tif"%(Var, unit, Date.year, Date.month, Date.day, Hour))

        if TimeStep == "daily":
            IDz_start = int(((Date - pd.Timestamp("2017-12-01")).days) * 8)
            IDz_end = IDz_start + 7
            output_name = os.path.join(output_folder, "%s_GEOS_%s_daily_%d.%02d.%02d.tif"%(Var, unit, Date.year, Date.month, Date.day))


        if not os.path.exists(output_name):

            # define total url
            url_start = r"https://opendap.nccs.nasa.gov/dods/GEOS-5/fp/0.25_deg/assim/inst3_2d_asm_Nx."
            url_GEOS = url_start + 'ascii?%s[%s:1:%s][%s:1:%s][%s:1:%s]' %(Var, IDz_start,IDz_end, int(IDy[0]),int(IDy[1]),int(IDx[0]),int(IDx[1]))

            # Reset the begin parameters for downloading
            downloaded = 0
            N = 0

            # if not downloaded try to download file
            while downloaded == 0:
                # try:

                # download data (first save as text file)
                pathtext = os.path.join(output_folder,'temp%s.txt' %str(IDz_start))

                # Download the data
                urllib.request.urlretrieve(url_GEOS, filename=pathtext)

                # Reshape data
                datashape = [int(IDy[1] - IDy[0] + 1), int(IDx[1] - IDx[0] + 1)]
                data_start = np.genfromtxt(pathtext,dtype = float,skip_header = 1,skip_footer = 6,delimiter = ',')
                data_list = np.asarray(data_start[:,1:])
                if TimeStep == "daily":
                    data_end = np.resize(data_list,(8, datashape[0], datashape[1]))
                if TimeStep == "three_hourly":
                    data_end = np.resize(data_list,(datashape[0], datashape[1]))
                os.remove(pathtext)

                # Set no data value
                data_end[data_end>1000000] = -9999

                if TimeStep == "daily":
                    if Var == "t2m":
                        data_end_max = np.nanmax(data_end, 0)
                        data_end_min = np.nanmin(data_end, 0)
                    if types == "state":
                        data_end = np.nanmean(data_end, 0)
                    else:
                        data_end = np.nansum(data_end, 0)

                # Add the VarFactor
                if VarInfo.factors[Var] < 0:
                    data_end[data_end != -9999] = data_end[data_end != -9999] + VarInfo.factors[Var]
                else:
                    data_end[data_end != -9999] = data_end[data_end != -9999] * VarInfo.factors[Var]
                data_end[data_end < -9999] = -9999

                # Download was succesfull
                downloaded = 1

                # twist the data
                data_end = np.flipud(data_end)

                # Save as tiff file
                PF.Save_as_tiff(output_name, data_end, geo_out, proj)
                all_files[Var].append(output_name)
                
                if TimeStep == "daily" and Var == "t2m":
                    # Add the VarFactor
                    if VarInfo.factors[Var] < 0:
                        data_end_max[data_end != -9999] = data_end_max[data_end != -9999] + VarInfo.factors[Var]
                        data_end_min[data_end != -9999] = data_end_min[data_end != -9999] + VarInfo.factors[Var]
                    else:
                        data_end_max[data_end != -9999] = data_end_max[data_end != -9999] * VarInfo.factors[Var]
                        data_end_min[data_end != -9999] = data_end_min[data_end != -9999] * VarInfo.factors[Var]
                    data_end_max[data_end < -9999] = -9999
                    data_end_min[data_end < -9999] = -9999
                    
                    # twist the data
                    data_end_min = np.flipud(data_end_min)
                    data_end_max = np.flipud(data_end_max)

                    # Save as tiff file
                    output_name = os.path.join(output_folder, "%s_GEOS_%s_daily-max_%d.%02d.%02d.tif"%(Var, unit, Date.year, Date.month, Date.day))
                    PF.Save_as_tiff(output_name, data_end_max, geo_out, proj)
                    all_files["t2m-max"].append(output_name)
                    output_name = os.path.join(output_folder, "%s_GEOS_%s_daily-min_%d.%02d.%02d.tif"%(Var, unit, Date.year, Date.month, Date.day))
                    PF.Save_as_tiff(output_name, data_end_min, geo_out, proj)
                    all_files["t2m-min"].append(output_name)

                # If download was not succesfull
                # except:

                #     # Try another time
                #     N = N + 1

                #     # Stop trying after 10 times
                #     if N == 10:
                #         print('Data from ' + Date.strftime('%Y-%m-%d') + ' is not available')
                #         downloaded = 1
        else:
            if os.path.isfile(output_name):
                all_files[Var].append(output_name)
            if TimeStep == "daily" and Var == "t2m":
                output_name = os.path.join(output_folder, "%s_GEOS_%s_daily-max_%d.%02d.%02d.tif"%(Var, unit, Date.year, Date.month, Date.day))
                if os.path.isfile(output_name):
                    all_files["t2m-max"].append(output_name)
                output_name = os.path.join(output_folder, "%s_GEOS_%s_daily-min_%d.%02d.%02d.tif"%(Var, unit, Date.year, Date.month, Date.day))
                if os.path.isfile(output_name):
                    all_files["t2m-min"].append(output_name)

        if Waitbar:
            Waitbar.update(1)
            # amount += 1
            # WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    return all_files

class VariablesInfo:
    """
    This class contains the information about the GEOS variables
    """
    names = {'t2m': 'Air_Temperature',
             'u2m': 'Eastward_Wind',
             'v2m': 'Northward_Wind',
             'qv2m': 'Specific_Humidity',
             'tqv': 'Total_Precipitable_Water_Vapor',
             'ps': 'Surface_Pressure',
             'slp': 'Sea_Level_Pressure'}

    descriptions = {'t2m': '2m Air Temperature',
             'u2m': '2m Eastward wind',
             'v2m': '2m Northward wind',
             'qv2m': '2m Specific Humidity',
             'tqv': 'Total Precipitable Water Vapor',
             'ps': 'Surface Pressure',
             'slp': 'Sea Level Pressure'}

    factors = {'t2m': 1,
             'u2m': 1,
             'v2m': 1,
             'qv2m': 1,
             'tqv': 1,
             'ps': 0.001,
             'slp':  0.001}

    types = {'t2m': 'state',
             'u2m': 'state',
             'v2m': 'state',
             'qv2m': 'state',
             'tqv': 'state',
             'ps': 'state',
             'slp': 'state'}

    def __init__(self, step):
        if step == 'three_hourly':
            self.units = {'t2m': 'K',
             'u2m': 'm-s-1',
             'v2m': 'm-s-1',
             'qv2m': 'kg-kg-1',
             'tqv': 'mm',
             'ps': 'kpa',
             'slp': 'kpa'}

        elif step == 'daily':
            self.units = {'t2m': 'K',
             'u2m': 'm-s-1',
             'v2m': 'm-s-1',
             'qv2m': 'kg-kg-1',
             'tqv': 'mm',
             'ps': 'kpa',
             'slp': 'kpa'}

        else:
            raise KeyError("The input time step is not supported")




# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MOD04

Description:
This module downloads MOD04 AOD550 data from
https://ladsweb.modaps.eosdis.nasa.gov. Use the MOD04.AOD550 function to
download and create daily AOD550 images in Gtiff format.
The data is available between 2000-02-18 till present.

Examples:
from pyWAPOR.Collect import MOD04
MOD04.AOD550(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2003-12-20',
           latlim=[41, 45], lonlim=[-8, -5])
"""

from .AOD550 import main as AOD550

__all__ = ['AOD550']

__version__ = '0.1'
